# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014-2016 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from gi.repository import Gio
from time import sleep
from testtools.matchers import Equals, NotEquals
from autopilot.matchers import Eventually

from lomiri_system_settings.tests import (
    SecurityBaseTestCase,
    SIM_IFACE)

from lomiri_system_settings.utils.i18n import ugettext as _
from lomiriuitoolkit import emulators as toolkit_emulators


class SecurityTestCase(SecurityBaseTestCase):
    """ Tests for Security Page """

    def setUp(self):
        super(SecurityTestCase, self).setUp()
        prps = self.security_page.get_properties()
        self.use_powerd = prps['usePowerd']
        if self.use_powerd:
            gsettings = Gio.Settings.new('com.lomiri.touch.system')
            prev = gsettings.get_uint('activity-timeout')
            self.addCleanup(
                self.set_prev_activity_timeout, gsettings, prev)
        else:
            gsettings = Gio.Settings.new('org.gnome.desktop.session')
            prev = gsettings.get_uint('idle-delay')
            self.addCleanup(
                self.set_prev_idle_delay, gsettings, prev)

    def set_prev_idle_delay(self, gsettings, prev):
        gsettings.set_uint('idle-delay', prev)
        self.assertThat(
            lambda: int(gsettings.get_uint('idle-delay')),
            Eventually(Equals(prev))
        )

    def set_prev_activity_timeout(self, gsettings, prev):
        gsettings.set_uint('activity-timeout', prev)
        self.assertThat(
            lambda: int(gsettings.get_uint('activity-timeout')),
            Eventually(Equals(prev))
        )

    def _get_activity_timeout(self):
        if self.use_powerd:
            gsettings = Gio.Settings.new('com.lomiri.touch.system')
            prev = gsettings.get_uint('activity-timeout')
            return prev
        else:
            gsettings = Gio.Settings.new('org.gnome.desktop.session')
            prev = gsettings.get_uint('idle-delay')
            return prev

    def _get_dim_timeout(self):
        if self.use_powerd:
            gsettings = Gio.Settings.new('com.lomiri.touch.system')
            prev = gsettings.get_uint('dim-timeout')
            return prev
        else:
            return None

    def _go_to_phone_lock(self):
        selector = self.security_page.select_single(
            objectName='lockingControl'
        )
        self.main_view.scroll_to_and_click(selector)

    def _go_to_sleep_values(self):
        self._go_to_phone_lock()
        selector = self.main_view.select_single(
            objectName='lockTimeout'
        )
        self.main_view.scroll_to_and_click(selector)

    def _get_sleep_selector(self):
        self._go_to_sleep_values()
        sleep_values_page = self.main_view.select_single(
            objectName='sleepValues'
        )
        self.assertThat(
            sleep_values_page,
            NotEquals(None)
        )
        self._go_to_sleep_values()
        sleep_values_page = self.main_view.select_single(
            objectName='sleepValues'
        )
        return sleep_values_page.select_single(
            toolkit_emulators.ItemSelector,
            objectName='sleepSelector'
        )

    def test_security_page(self):
        """ Checks whether Security page is available """
        self.assertThat(
            self.security_page,
            NotEquals(None)
        )
        self.assertThat(
            self.security_page.title,
            Equals(_('Security & Privacy'))
        )

    def test_locking_control_value(self):
        self._go_to_phone_lock()
        actTimeout = self._get_activity_timeout()
        dimTimeout = self._get_dim_timeout()
        activityTimeout = self.main_view.select_single(
            objectName='lockTimeout').value
        if actTimeout is 0:
            self.assertEquals(activityTimeout, ('Manually'))
        elif actTimeout is 60:
            self.assertEquals(
                activityTimeout,
                ('{:d} minute').format(int(actTimeout/60)))
            if dimTimeout:
                self.assertEquals(dimTimeout, actTimeout - 10)
        else:
            self.assertEquals(
                activityTimeout,
                ('After {:d} minutes').format(int(actTimeout/60)))
            if dimTimeout:
                self.assertEquals(dimTimeout, actTimeout - 10)

    def test_phone_lock_page(self):
        self._go_to_phone_lock()
        phone_lock_page = self.main_view.select_single(
            objectName='phoneLockingPage'
        )
        self.assertThat(
            phone_lock_page,
            NotEquals(None)
        )
        self.assertThat(
            phone_lock_page.title,
            Equals(_('Locking and unlocking'))
        )

    def test_lock_security_focus_on_entry(self):
        self._go_to_phone_lock()

        phone_lock_page = self.main_view.select_single(
            objectName='phoneLockingPage')
        selector = phone_lock_page.select_single(objectName='lockSecurity')
        self.main_view.scroll_to_and_click(selector)

        lock_security_page = self.main_view.wait_select_single(
            objectName='lockSecurityPage')

        # Find the selected security method.
        unlock_methods = ['method_swipe', 'method_code', 'method_phrase']
        selected_method = None
        for m in unlock_methods:
            if lock_security_page.wait_select_single(objectName=m).selected:
                selected_method = m

        # If swipe is the selected security, we trigger the dialog by
        # changing the security to method_code
        if selected_method == 'method_swipe':
            dialog_trigger = lock_security_page.wait_select_single(
                objectName='method_code')
            input_selector = 'newInput'
        else:
            # If the security is anything besides swipe, we trigger the current
            # pass dialog by changing the security to swipe.
            dialog_trigger = lock_security_page.wait_select_single(
                objectName='method_swipe')
            input_selector = 'currentInput'

        # Trigger dialog.
        self.main_view.scroll_to_and_click(dialog_trigger)

        # Find the text input.
        dialog = self.main_view.wait_select_single(
            objectName='changeSecurityDialog')
        text_input = dialog.wait_select_single(
            objectName=input_selector)
        self.assertTrue(text_input.focus)

    def test_phone_lock_value(self):
        self._go_to_phone_lock()
        phone_lock_page = self.main_view.select_single(
            objectName='phoneLockingPage'
        )
        actTimeout = self._get_activity_timeout()
        activityTimeout = phone_lock_page.select_single(
            objectName='lockTimeout').value
        if actTimeout is 0:
            self.assertEquals(activityTimeout, ('Never'))
        elif actTimeout is 60:
            self.assertEquals(
                activityTimeout,
                ('After {:d} minute').format(int(actTimeout/60))
            )
        else:
            self.assertEquals(
                activityTimeout,
                ('After {:d} minutes').format(int(actTimeout/60))
            )

    def test_idle_never_timeout(self):
        selector = self._get_sleep_selector()
        to_select = selector.select_single(
            'OptionSelectorDelegate', text='Never')
        self.main_view.pointing_device.click_object(to_select)
        to_select.selected.wait_for(True)
        sleep(1)
        actTimeout = self._get_activity_timeout()
        self.assertEquals(actTimeout, 0)
        selected_delegate = selector.select_single(
            'OptionSelectorDelegate', selected=True)
        self.assertEquals(selected_delegate.text, 'Never')

    def test_idle_change_timeout(self):
        selector = self._get_sleep_selector()
        to_select = selector.select_single(
            'OptionSelectorDelegate', text='After 4 minutes')
        self.main_view.pointing_device.click_object(to_select)
        to_select.selected.wait_for(True)
        sleep(1)
        actTimeout = self._get_activity_timeout()
        self.assertEquals(actTimeout, 240)
        selected_delegate = selector.select_single(
            'OptionSelectorDelegate', selected=True)
        self.assertEquals(selected_delegate.text, 'After 4 minutes')

    def test_sim_pin_control_value(self):
        self.assertEqual('none', self.modem_0.Get(SIM_IFACE, 'PinRequired'))
        self.assertEqual(['pin'], self.modem_0.Get(SIM_IFACE, 'LockedPins'))

        sim_pin_value = self.security_page.select_single(
            objectName='simControl').value

        self.assertThat(
            sim_pin_value,
            Equals(_('On'))
        )


class SecuritySimPinLockedTestCase(SecurityBaseTestCase):
    """ Tests for Security Page with Locked SIM Pin """

    def setUp(self):
        super(SecuritySimPinLockedTestCase, self).setUp()
        self.modem_0.LockPin('pin', '2468')
        self.assertEqual(['pin'], self.modem_0.Get(SIM_IFACE, 'LockedPins'))

    def test_sim_pin_lock_control(self):
        sim_pin_page = self.security_page.go_to_sim_lock()

        switch = sim_pin_page.get_sim_pin_switch(0)
        locked = len(self.modem_0.Get(SIM_IFACE, 'LockedPins')) > 0
        self.assertEquals(locked, switch.checked)

    def test_sim_pin_lock_control_unlock(self):
        sim_pin_page = self.security_page.go_to_sim_lock()
        switch = sim_pin_page.get_sim_pin_switch(0)

        self.assertTrue(switch.checked)

        sim_pin_page.click_sim_pin_switch(0)

        lock_dialog = self.main_view.select_single(
            objectName='lockDialogComponent'
        )
        self.assertEqual(
            lock_dialog.title,
            _("Enter SIM PIN")
        )

        prev_input = self.main_view.select_single(
            objectName='prevInput'
        )
        submit_button = self.main_view.select_single(
            objectName='lockButton'
        )

        self.assertEqual(
            submit_button.text,
            _("Unlock")
        )

        self.assertFalse(
            submit_button.enabled
        )
        self.main_view.scroll_to_and_click(prev_input)
        prev_input.write("246")
        self.assertFalse(
            submit_button.enabled
        )
        prev_input.write("2468")

        self.assertTrue(
            submit_button.enabled
        )

        self.main_view.scroll_to_and_click(submit_button)

        self.assertFalse(switch.checked)

        locked = len(self.modem_0.Get(SIM_IFACE, 'LockedPins')) > 0
        self.assertEquals(locked, switch.checked)

    def test_sim_pin_lock_control_unlock_fail(self):
        sim_pin_page = self.security_page.go_to_sim_lock()
        switch = sim_pin_page.get_sim_pin_switch(0)

        self.assertTrue(
            len(self.modem_0.Get(SIM_IFACE, 'LockedPins')) > 0
        )
        self.assertTrue(switch.checked)

        sim_pin_page.click_sim_pin_switch(0)

        lock_dialog = self.main_view.select_single(
            objectName='lockDialogComponent'
        )
        self.assertEqual(
            lock_dialog.title,
            _("Enter SIM PIN")
        )

        prev_input = self.main_view.select_single(
            objectName='prevInput'
        )
        submit_button = self.main_view.select_single(
            objectName='lockButton'
        )

        self.assertThat(
            submit_button.text,
            Eventually(Equals(_("Unlock")))
        )

        self.assertFalse(
            submit_button.enabled
        )

        self.main_view.scroll_to_and_click(prev_input)
        prev_input.write("1234")

        self.assertTrue(
            submit_button.enabled
        )

        self.main_view.scroll_to_and_click(submit_button)

        self.assertTrue(
            len(self.modem_0.Get(SIM_IFACE, 'LockedPins')) > 0
        )


class SecuritySimPinUnlockedTestCase(SecurityBaseTestCase):
    """ Tests for Security Page with Unlocked SIM Pin """

    def setUp(self):
        super(SecuritySimPinUnlockedTestCase, self).setUp()
        self.modem_0.UnlockPin('pin', '2468')
        self.assertFalse(len(self.modem_0.Get(SIM_IFACE, 'LockedPins')) > 0)

    def test_sim_pin_lock_control_lock(self):
        sim_pin_page = self.security_page.go_to_sim_lock()
        switch = sim_pin_page.get_sim_pin_switch(0)

        self.assertFalse(switch.checked)

        sim_pin_page.click_sim_pin_switch(0)

        lock_dialog = self.main_view.select_single(
            objectName='lockDialogComponent'
        )
        self.assertEqual(
            lock_dialog.title,
            _("Enter Previous SIM PIN")
        )

        prev_input = self.main_view.select_single(
            objectName='prevInput'
        )
        submit_button = self.main_view.select_single(
            objectName='lockButton'
        )

        self.assertEqual(
            submit_button.text,
            _("Lock")
        )

        self.assertFalse(
            submit_button.enabled
        )
        self.main_view.scroll_to_and_click(prev_input)
        prev_input.write("246")
        self.assertFalse(
            submit_button.enabled
        )
        prev_input.write("2468")

        self.assertTrue(
            submit_button.enabled
        )

        self.main_view.scroll_to_and_click(submit_button)

        self.assertTrue(switch.checked)

        self.assertEqual(['pin'], self.modem_0.Get(SIM_IFACE, 'LockedPins'))
        locked = len(self.modem_0.Get(SIM_IFACE, 'LockedPins')) > 0
        self.assertEquals(locked, switch.checked)

    def test_sim_pin_lock_control_lock_fail(self):
        sim_pin_page = self.security_page.go_to_sim_lock()

        self.assertFalse(
            len(self.modem_0.Get(SIM_IFACE, 'LockedPins')) > 0
        )

        sim_pin_page.click_sim_pin_switch(0)

        lock_dialog = self.main_view.select_single(
            objectName='lockDialogComponent'
        )
        self.assertEqual(
            lock_dialog.title,
            _("Enter Previous SIM PIN")
        )

        prev_input = self.main_view.select_single(
            objectName='prevInput'
        )
        submit_button = self.main_view.select_single(
            objectName='lockButton'
        )

        self.assertThat(
            submit_button.text,
            Eventually(Equals(_("Lock")))
        )

        self.assertFalse(
            submit_button.enabled
        )
        self.main_view.scroll_to_and_click(prev_input)
        prev_input.write("1234")

        self.assertTrue(
            submit_button.enabled
        )

        self.main_view.scroll_to_and_click(submit_button)

        self.assertFalse(
            len(self.modem_0.Get(SIM_IFACE, 'LockedPins')) > 0
        )



'''NetworkManager mock template

This creates the expected methods and properties of the main
org.freedesktop.NetworkManager object, but no devices. You can specify any
property such as 'NetworkingEnabled', or 'WirelessEnabled' etc. in
"parameters".
'''

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.  See http://www.gnu.org/copyleft/lgpl.html for the full text
# of the license.

from dbusmock import MOCK_IFACE
import binascii
import dbus
import dbusmock
import uuid

__author__ = 'Iftikhar Ahmad'
__email__ = 'iftikhar.ahmad@canonical.com'
__copyright__ = '(c) 2012 Canonical Ltd.'
__license__ = 'LGPL 3+'

BUS_NAME = 'org.freedesktop.NetworkManager'
MAIN_OBJ = '/org/freedesktop/NetworkManager'
MAIN_IFACE = 'org.freedesktop.NetworkManager'
SETTINGS_OBJ = '/org/freedesktop/NetworkManager/Settings'
SETTINGS_IFACE = 'org.freedesktop.NetworkManager.Settings'
DEVICE_IFACE = 'org.freedesktop.NetworkManager.Device'
WIRELESS_DEVICE_IFACE = 'org.freedesktop.NetworkManager.Device.Wireless'
ACCESS_POINT_IFACE = 'org.freedesktop.NetworkManager.AccessPoint'
CSETTINGS_IFACE = 'org.freedesktop.NetworkManager.Settings.Connection'
ACTIVE_CONNECTION_IFACE = 'org.freedesktop.NetworkManager.Connection.Active'
ACTIVE_CONNECTION_PATH = '/org/freedesktop/NetworkManager/ActiveConnection/'
SYSTEM_BUS = True


class NMState:
    '''Global state

    As per https://developer.gnome.org/NetworkManager/unstable/spec.html
    #type-NM_STATE
    '''
    NM_STATE_UNKNOWN = 0
    NM_STATE_ASLEEP = 10
    NM_STATE_DISCONNECTED = 20
    NM_STATE_DISCONNECTING = 30
    NM_STATE_CONNECTING = 40
    NM_STATE_CONNECTED_LOCAL = 50
    NM_STATE_CONNECTED_SITE = 60
    NM_STATE_CONNECTED_GLOBAL = 70


class NMConnectivityState:
    '''Connectvity state

    As per https://developer.gnome.org/NetworkManager/unstable/spec.html
    #type-NM_CONNECTIVITY
    '''
    NM_CONNECTIVITY_UNKNOWN = 0
    NM_CONNECTIVITY_NONE = 1
    NM_CONNECTIVITY_PORTAL = 2
    NM_CONNECTIVITY_LIMITED = 3
    NM_CONNECTIVITY_FULL = 4


class NMActiveConnectionState:
    '''Active connection state

    As per https://developer.gnome.org/NetworkManager/unstable/spec.html
    #type-NM_ACTIVE_CONNECTION_STATE
    '''
    NM_ACTIVE_CONNECTION_STATE_UNKNOWN = 0
    NM_ACTIVE_CONNECTION_STATE_ACTIVATING = 1
    NM_ACTIVE_CONNECTION_STATE_ACTIVATED = 2
    NM_ACTIVE_CONNECTION_STATE_DEACTIVATING = 3
    NM_ACTIVE_CONNECTION_STATE_DEACTIVATED = 4


class InfrastructureMode:
    '''Infrastructure mode

    As per https://developer.gnome.org/NetworkManager/unstable/spec.html
    #type-NM_802_11_MODE
    '''
    NM_802_11_MODE_UNKNOWN = 0
    NM_802_11_MODE_ADHOC = 1
    NM_802_11_MODE_INFRA = 2
    NM_802_11_MODE_AP = 3

    NAME_MAP = {
        NM_802_11_MODE_UNKNOWN: 'unknown',
        NM_802_11_MODE_ADHOC: 'adhoc',
        NM_802_11_MODE_INFRA: 'infrastructure',
        NM_802_11_MODE_AP: 'access-point',
    }


class DeviceState:
    '''Device states

    As per https://developer.gnome.org/NetworkManager/unstable/spec.html
    #type-NM_DEVICE_STATE
    '''
    UNKNOWN = 0
    UNMANAGED = 10
    UNAVAILABLE = 20
    DISCONNECTED = 30
    PREPARE = 40
    CONFIG = 50
    NEED_AUTH = 60
    IP_CONFIG = 70
    IP_CHECK = 80
    SECONDARIES = 90
    ACTIVATED = 100
    DEACTIVATING = 110
    FAILED = 120


class NM80211ApSecurityFlags:
    '''Security flags

    As per https://developer.gnome.org/NetworkManager/unstable/spec.html
    #type-NM_802_11_AP_SEC
    '''
    NM_802_11_AP_SEC_NONE = 0x00000000
    NM_802_11_AP_SEC_PAIR_WEP40 = 0x00000001
    NM_802_11_AP_SEC_PAIR_WEP104 = 0x00000002
    NM_802_11_AP_SEC_PAIR_TKIP = 0x00000004
    NM_802_11_AP_SEC_PAIR_CCMP = 0x00000008
    NM_802_11_AP_SEC_GROUP_WEP40 = 0x00000010
    NM_802_11_AP_SEC_GROUP_WEP104 = 0x00000020
    NM_802_11_AP_SEC_GROUP_TKIP = 0x00000040
    NM_802_11_AP_SEC_GROUP_CCMP = 0x00000080
    NM_802_11_AP_SEC_KEY_MGMT_PSK = 0x00000100
    NM_802_11_AP_SEC_KEY_MGMT_802_1X = 0x00000200

    NAME_MAP = {
        NM_802_11_AP_SEC_KEY_MGMT_PSK: {
            'key-mgmt': 'wpa-psk',
            'auth-alg': 'open'
        },
        NM_802_11_AP_SEC_KEY_MGMT_802_1X: {
            'key-mgmt': 'wpa-eap'
        },
    }


class NM80211ApFlags:
    '''Device flags

    As per https://developer.gnome.org/NetworkManager/unstable/spec.html
    #type-NM_802_11_AP_FLAGS
    '''
    NM_802_11_AP_FLAGS_NONE = 0x00000000
    NM_802_11_AP_FLAGS_PRIVACY = 0x00000001


def activate_connection(self, conn, dev, ap):
    name = ap.rsplit('/', 1)[1]
    RemoveActiveConnection(self, dev, ACTIVE_CONNECTION_PATH + name)

    state = dbus.UInt32(
        NMActiveConnectionState.NM_ACTIVE_CONNECTION_STATE_ACTIVATED
    )
    active_conn = dbus.ObjectPath(
        AddActiveConnection(self, [dev], conn, ap, name, state)
    )

    return active_conn


def deactivate_connection(self, active_conn_path):
    NM = dbusmock.get_object(MAIN_OBJ)

    for dev_path in NM.GetDevices():
        RemoveActiveConnection(self, dev_path, active_conn_path)


def add_and_activate_connection(self, conn_conf, dev, ap):
    name = ap.rsplit('/', 1)[1]
    RemoveWifiConnection(
        self, dev, '/org/freedesktop/NetworkManager/Settings/' + name
    )

    raw_ssid = ''.join(
        [chr(byte) for byte in conn_conf["802-11-wireless"]["ssid"]]
    )
    wifi_conn = dbus.ObjectPath(
        AddWiFiConnection(self, dev, name, raw_ssid, "", conn_conf)
    )

    active_conn = activate_connection(self, wifi_conn, dev, ap)

    return (wifi_conn, active_conn)


def load(mock, parameters):
    mock.activate_connection = activate_connection
    mock.deactivate_connection = deactivate_connection
    mock.add_and_activate_connection = add_and_activate_connection
    mock.AddMethods(MAIN_IFACE, [
        (
            'GetDevices', '', 'ao',
            'ret = [k for k in objects.keys() if "/Devices" in k]'
        ),
        ('GetPermissions', '', 'a{ss}', 'ret = {}'),
        ('state', '', 'u', "ret = self.Get('%s', 'State')" % MAIN_IFACE),
        (
            'CheckConnectivity', '', 'u',
            "ret = self.Get('%s', 'Connectivity')" % MAIN_IFACE
        ),
        (
            'ActivateConnection', 'ooo', 'o',
            "ret = self.activate_connection(self, args[0], args[1], args[2])"
        ),
        (
            'DeactivateConnection', 'o', '',
            "self.deactivate_connection(self, args[0])"
        ),
        (
            'AddAndActivateConnection', 'a{sa{sv}}oo', 'oo',
            "ret = self.add_and_activate_connection(" +
            "self, args[0], args[1], args[2])"
        ),
    ])

    mock.AddProperties(
        '',
        {
            'ActiveConnections': dbus.Array([], signature='o'),
            'Devices': dbus.Array([], signature='o'),
            'NetworkingEnabled': parameters.get('NetworkingEnabled', True),
            'Connectivity': parameters.get(
                'Connectivity',
                dbus.UInt32(NMConnectivityState.NM_CONNECTIVITY_FULL)
            ),
            'State': parameters.get(
                'State', dbus.UInt32(NMState.NM_STATE_CONNECTED_GLOBAL)
            ),
            'Startup': False,
            'Version': parameters.get('Version', '0.9.6.0'),
            'WimaxEnabled': parameters.get('WimaxEnabled', True),
            'WimaxHardwareEnabled': parameters.get(
                'WimaxHardwareEnabled', True
            ),
            'WirelessEnabled': parameters.get('WirelessEnabled', True),
            'WirelessHardwareEnabled': parameters.get(
                'WirelessHardwareEnabled', True
            ),
            'WwanEnabled': parameters.get('WwanEnabled', False),
            'WwanHardwareEnabled': parameters.get('WwanHardwareEnabled', True)
        }
    )

    settings_props = {'Hostname': 'hostname',
                      'CanModify': True,
                      'Connections': dbus.Array([], signature='o')}
    settings_methods = [
        (
            'ListConnections', '', 'ao',
            "ret = self.Get('%s', 'Connections')" % SETTINGS_IFACE
        ),
        ('GetConnectionByUuid', 's', 'o', ''),
        (
            'AddConnection', 'a{sa{sv}}', 'o',
            'ret = self.SettingsAddConnection(args[0])'
        ),
        ('SaveHostname', 's', '', '')
    ]
    mock.AddObject(SETTINGS_OBJ,
                   SETTINGS_IFACE,
                   settings_props,
                   settings_methods)


@dbus.service.method(MOCK_IFACE,
                     in_signature='sssv', out_signature='')
def SetProperty(self, path, iface, name, value):
    obj = dbusmock.get_object(path)
    obj.Set(iface, name, value)
    obj.EmitSignal(iface, 'PropertiesChanged', 'a{sv}', [{name: value}])


@dbus.service.method(MOCK_IFACE,
                     in_signature='u', out_signature='')
def SetGlobalConnectionState(self, state):
    self.SetProperty(
        MAIN_OBJ, MAIN_IFACE, 'State',
        dbus.UInt32(state, variant_level=1)
    )
    self.EmitSignal(MAIN_IFACE, 'StateChanged', 'u', [state])


@dbus.service.method(MOCK_IFACE,
                     in_signature='u', out_signature='')
def SetConnectivity(self, connectivity):
    self.SetProperty(
        MAIN_OBJ, MAIN_IFACE, 'Connectivity',
        dbus.UInt32(connectivity, variant_level=1)
    )


@dbus.service.method(MOCK_IFACE,
                     in_signature='ss', out_signature='')
def SetDeviceActive(self, device_path, active_connection_path):
    dev_obj = dbusmock.get_object(device_path)
    dev_obj.Set(
        DEVICE_IFACE, 'ActiveConnection',
        dbus.ObjectPath(active_connection_path)
    )
    old_state = dev_obj.Get(DEVICE_IFACE, 'State')
    dev_obj.Set(DEVICE_IFACE, 'State', dbus.UInt32(DeviceState.ACTIVATED))

    dev_obj.EmitSignal(
        DEVICE_IFACE, 'StateChanged', 'uuu',
        [
            dbus.UInt32(DeviceState.ACTIVATED),
            old_state,
            dbus.UInt32(1)
        ]
    )


@dbus.service.method(MOCK_IFACE,
                     in_signature='s', out_signature='')
def SetDeviceDisconnected(self, device_path):
    dev_obj = dbusmock.get_object(device_path)
    dev_obj.Set(DEVICE_IFACE, 'ActiveConnection', dbus.ObjectPath('/'))
    old_state = dev_obj.Get(DEVICE_IFACE, 'State')
    dev_obj.Set(DEVICE_IFACE, 'State', dbus.UInt32(DeviceState.DISCONNECTED))

    dev_obj.EmitSignal(
        DEVICE_IFACE, 'StateChanged', 'uuu',
        [
            dbus.UInt32(DeviceState.DISCONNECTED),
            old_state,
            dbus.UInt32(1)
        ]
    )


@dbus.service.method(MOCK_IFACE,
                     in_signature='ssi', out_signature='s')
def AddEthernetDevice(self, device_name, iface_name, state):
    '''Add an ethernet device.

    You have to specify device_name, device interface name (e. g. eth0), and
    state. You can use the predefined DeviceState values (e. g.
    DeviceState.ACTIVATED) or supply a numeric value. For valid state values
    please visit
    http://projects.gnome.org/NetworkManager/developers/api/09/spec.html
    #type-NM_DEVICE_STATE

    Please note that this does not set any global properties.

    Returns the new object path.
    '''
    path = '/org/freedesktop/NetworkManager/Devices/' + device_name
    wired_props = {'Carrier': False,
                   'HwAddress': dbus.String('78:DD:08:D2:3D:43'),
                   'PermHwAddress': dbus.String('78:DD:08:D2:3D:43'),
                   'Speed': dbus.UInt32(0)}
    self.AddObject(path,
                   'org.freedesktop.NetworkManager.Device.Wired',
                   wired_props,
                   [])

    props = {'DeviceType': dbus.UInt32(1),
             'State': dbus.UInt32(state),
             'Interface': iface_name,
             'AvailableConnections': dbus.Array([], signature='o'),
             'IpInterface': ''}

    obj = dbusmock.get_object(path)
    obj.AddProperties(DEVICE_IFACE, props)

    devices = self.Get(MAIN_IFACE, 'Devices')
    devices.append(path)
    self.Set(MAIN_IFACE, 'Devices', devices)

    self.EmitSignal(
        'org.freedesktop.NetworkManager', 'DeviceAdded', 'o', [path]
    )

    return path


@dbus.service.method(MOCK_IFACE,
                     in_signature='ssi', out_signature='s')
def AddWiFiDevice(self, device_name, iface_name, state):
    '''Add a WiFi Device.

    You have to specify device_name, device interface name (e. g.  wlan0) and
    state. You can use the predefined DeviceState values (e. g.
    DeviceState.ACTIVATED) or supply a numeric value. For valid state values,
    please visit
    http://projects.gnome.org/NetworkManager/developers/api/09/spec.html
    #type-NM_DEVICE_STATE

    Please note that this does not set any global properties.

    Returns the new object path.
    '''

    path = '/org/freedesktop/NetworkManager/Devices/' + device_name
    self.AddObject(path,
                   WIRELESS_DEVICE_IFACE,
                   {
                       'HwAddress': dbus.String('11:22:33:44:55:66'),
                       'PermHwAddress': dbus.String('11:22:33:44:55:66'),
                       'Bitrate': dbus.UInt32(5400),
                       'Mode': dbus.UInt32(2),
                       'WirelessCapabilities': dbus.UInt32(255),
                       'AccessPoints': dbus.Array([], signature='o'),
                   },
                   [
                       ('GetAccessPoints', '', 'ao',
                        'ret = self.access_points'),
                       ('GetAllAccessPoints', '', 'ao',
                        'ret = self.access_points'),
                       ('RequestScan', 'a{sv}', '', ''),
                   ])

    dev_obj = dbusmock.get_object(path)
    dev_obj.access_points = []
    dev_obj.AddProperties(
        DEVICE_IFACE,
        {
            'ActiveConnection': dbus.ObjectPath('/'),
            'AvailableConnections': dbus.Array([], signature='o'),
            'AutoConnect': False,
            'Managed': True,
            'Driver': 'dbusmock',
            'DeviceType': dbus.UInt32(2),
            'State': dbus.UInt32(state),
            'Interface': iface_name,
            'IpInterface': iface_name,
        }
    )

    devices = self.Get(MAIN_IFACE, 'Devices')
    devices.append(path)
    self.Set(MAIN_IFACE, 'Devices', devices)

    self.EmitSignal(
        'org.freedesktop.NetworkManager', 'DeviceAdded', 'o', [path]
    )

    return path


@dbus.service.method(MOCK_IFACE,
                     in_signature='ssssuuuyu', out_signature='s')
def AddAccessPoint(self, dev_path, ap_name, ssid, hw_address,
                   mode, frequency, rate, strength, security):
    '''Add an access point to an existing WiFi device.

    You have to specify WiFi Device path, Access Point object name,
    ssid, hw_address, mode, frequency, rate, strength and security.
    For valid access point property values, please visit
    http://projects.gnome.org/NetworkManager/developers/api/09/spec.html#
    org.freedesktop.NetworkManager.AccessPoint

    Please note that this does not set any global properties.

    Returns the new object path.
    '''
    dev_obj = dbusmock.get_object(dev_path)
    ap_path = '/org/freedesktop/NetworkManager/AccessPoint/' + ap_name
    if ap_path in dev_obj.access_points:
        raise dbus.exceptions.DBusException(
            'Access point %s on device %s already exists' % (ap_name,
                                                             dev_path),
            name=MAIN_IFACE + '.AlreadyExists')

    flags = NM80211ApFlags.NM_802_11_AP_FLAGS_PRIVACY
    if security == NM80211ApSecurityFlags.NM_802_11_AP_SEC_NONE:
        flags = NM80211ApFlags.NM_802_11_AP_FLAGS_NONE

    self.AddObject(ap_path,
                   ACCESS_POINT_IFACE,
                   {'Ssid': dbus.ByteArray(ssid.encode('UTF-8')),
                    'HwAddress': dbus.String(hw_address),
                    'Flags': dbus.UInt32(flags),
                    'LastSeen': dbus.Int32(1),
                    'Frequency': dbus.UInt32(frequency),
                    'MaxBitrate': dbus.UInt32(rate),
                    'Mode': dbus.UInt32(mode),
                    'RsnFlags': dbus.UInt32(security),
                    'WpaFlags': dbus.UInt32(security),
                    'Strength': dbus.Byte(strength)},
                   [])

    dev_obj.access_points.append(ap_path)

    aps = dev_obj.Get(WIRELESS_DEVICE_IFACE, 'AccessPoints')
    aps.append(ap_path)
    dev_obj.Set(WIRELESS_DEVICE_IFACE, 'AccessPoints', aps)

    dev_obj.EmitSignal(
        WIRELESS_DEVICE_IFACE, 'AccessPointAdded', 'o', [ap_path]
    )

    return ap_path


@dbus.service.method(MOCK_IFACE,
                     in_signature='ssssa{sa{sv}}', out_signature='s')
def AddWiFiConnection(self, dev_path, connection_name, ssid_name, key_mgmt,
                      config):
    '''Add an available connection to an existing WiFi device and access point.

    You have to specify WiFi Device path, Connection object name,
    SSID and key management.

    The SSID must match one of the previously created access points.

    Please note that this does not set any global properties.

    Returns the new object path.
    '''

    dev_obj = dbusmock.get_object(dev_path)
    connection_path = '%s/%s' % (SETTINGS_OBJ, connection_name)
    connections = dev_obj.Get(DEVICE_IFACE, 'AvailableConnections')

    settings_obj = dbusmock.get_object(SETTINGS_OBJ)
    main_connections = settings_obj.ListConnections()

    ssid = ssid_name.encode('UTF-8')

    # Find the access point by ssid
    access_point = None
    access_points = dev_obj.access_points
    for ap_path in access_points:
        ap = dbusmock.get_object(ap_path)
        if ap.Get(ACCESS_POINT_IFACE, 'Ssid') == ssid:
            access_point = ap
            break

    if not access_point:
        raise dbus.exceptions.DBusException(
            'Access point with SSID [%s] could not be found' % (ssid_name),
            name=MAIN_IFACE + '.DoesNotExist')

    hw_address = access_point.Get(ACCESS_POINT_IFACE, 'HwAddress')
    mode = access_point.Get(ACCESS_POINT_IFACE, 'Mode')
    security = access_point.Get(ACCESS_POINT_IFACE, 'WpaFlags')

    if connection_path in connections or connection_path in main_connections:
        raise dbus.exceptions.DBusException(
            'Connection %s on device %s already exists' % (
                connection_name, dev_path
            ),
            name=MAIN_IFACE + '.AlreadyExists')

    # Parse mac address string into byte array
    mac_bytes = binascii.unhexlify(hw_address.replace(':', ''))

    settings = {
        '802-11-wireless': {
            'seen-bssids': [hw_address],
            'ssid': dbus.ByteArray(ssid),
            'mac-address': dbus.ByteArray(mac_bytes),
            'mode': InfrastructureMode.NAME_MAP[mode]
        },
        'connection': {
            'timestamp': dbus.UInt64(1374828522),
            'type': '802-11-wireless',
            'id': ssid_name,
            'uuid': str(uuid.uuid4())
        },
    }

    if security != NM80211ApSecurityFlags.NM_802_11_AP_SEC_NONE:
        settings['802-11-wireless']['security'] = '802-11-wireless-security'
        settings['802-11-wireless-security'] = (
            NM80211ApSecurityFlags.NAME_MAP[security])

    if security == NM80211ApSecurityFlags.NM_802_11_AP_SEC_KEY_MGMT_802_1X:
        settings['802-1x'] = config['802-1x']

    self.AddObject(
        connection_path,
        CSETTINGS_IFACE,
        {
            'Settings': dbus.Dictionary(settings, signature='sa{sv}'),
            'Secrets': dbus.Dictionary({}, signature='sa{sv}'),
        },
        [
            (
                'Delete', '', '',
                'self.ConnectionDelete("%s")' % connection_path
            ),
            (
                'GetSettings', '', 'a{sa{sv}}',
                "ret = self.Get('%s', 'Settings')" % CSETTINGS_IFACE
            ),
            (
                'GetSecrets', 's', 'a{sa{sv}}',
                "ret = self.Get('%s', 'Secrets')" % CSETTINGS_IFACE
            ),
            (
                'Update', 'a{sa{sv}}', '',
                'self.ConnectionUpdate("%s", args[0])' % connection_path
            ),
        ]
    )

    connections.append(dbus.ObjectPath(connection_path))
    dev_obj.Set(DEVICE_IFACE, 'AvailableConnections', connections)

    main_connections.append(connection_path)
    settings_obj.Set(SETTINGS_IFACE, 'Connections', main_connections)

    settings_obj.EmitSignal(SETTINGS_IFACE, 'NewConnection', 'o', [ap_path])

    return connection_path


@dbus.service.method(MOCK_IFACE,
                     in_signature='assssu', out_signature='s')
def AddActiveConnection(self, devices, connection_device, specific_object,
                        name, state):
    '''Add an active connection to an existing WiFi device.

    You have to a list of the involved WiFi devices, the connection path,
    the access point path, ActiveConnection object name and connection
    state.

    Please note that this does not set any global properties.

    Returns the new object path.
    '''

    conn_obj = dbusmock.get_object(connection_device)
    settings = conn_obj.Get(CSETTINGS_IFACE, 'Settings')
    conn_uuid = settings['connection']['uuid']

    device_objects = [dbus.ObjectPath(dev) for dev in devices]

    active_connection_path = ACTIVE_CONNECTION_PATH + name
    self.AddObject(active_connection_path,
                   ACTIVE_CONNECTION_IFACE,
                   {
                       'Devices': device_objects,
                       'Default6': False,
                       'Default': True,
                       'Vpn': False,
                       'Connection': dbus.ObjectPath(connection_device),
                       'Master': dbus.ObjectPath('/'),
                       'SpecificObject': dbus.ObjectPath(specific_object),
                       'Uuid': conn_uuid,
                       'State': state,
                   },
                   [])

    for dev_path in devices:
        self.SetDeviceActive(dev_path, active_connection_path)

    active_connections = self.Get(MAIN_IFACE, 'ActiveConnections')
    active_connections.append(dbus.ObjectPath(active_connection_path))
    self.SetProperty(
        MAIN_OBJ, MAIN_IFACE, 'ActiveConnections', active_connections
    )

    return active_connection_path


@dbus.service.method(MOCK_IFACE,
                     in_signature='ss', out_signature='')
def RemoveAccessPoint(self, dev_path, ap_path):
    '''Remove the specified access point.

    You have to specify the device to remove the access point from, and the
    path of the access point.

    Please note that this does not set any global properties.
    '''

    dev_obj = dbusmock.get_object(dev_path)

    aps = dev_obj.Get(WIRELESS_DEVICE_IFACE, 'AccessPoints')
    aps.remove(ap_path)
    dev_obj.Set(WIRELESS_DEVICE_IFACE, 'AccessPoints', aps)

    dev_obj.access_points.remove(ap_path)

    dev_obj.EmitSignal(
        WIRELESS_DEVICE_IFACE, 'AccessPointRemoved', 'o', [ap_path]
    )

    self.RemoveObject(ap_path)


@dbus.service.method(MOCK_IFACE,
                     in_signature='ss', out_signature='')
def RemoveWifiConnection(self, dev_path, connection_path):
    '''Remove the specified WiFi connection.

    You have to specify the device to remove the connection from, and the
    path of the Connection.

    Please note that this does not set any global properties.
    '''

    dev_obj = dbusmock.get_object(dev_path)
    settings_obj = dbusmock.get_object(SETTINGS_OBJ)

    connections = dev_obj.Get(DEVICE_IFACE, 'AvailableConnections')
    main_connections = settings_obj.ListConnections()

    if (connection_path not in connections and
            connection_path not in main_connections):
        return

    connections.remove(dbus.ObjectPath(connection_path))
    dev_obj.Set(DEVICE_IFACE, 'AvailableConnections', connections)

    main_connections.remove(connection_path)
    settings_obj.Set(SETTINGS_IFACE, 'Connections', main_connections)

    settings_obj.EmitSignal(
        SETTINGS_IFACE, 'ConnectionRemoved', 'o', [connection_path]
    )

    connection_obj = dbusmock.get_object(connection_path)
    connection_obj.EmitSignal(CSETTINGS_IFACE, 'Removed', '', [])

    self.RemoveObject(connection_path)


@dbus.service.method(MOCK_IFACE,
                     in_signature='ss', out_signature='')
def RemoveActiveConnection(self, dev_path, active_connection_path):
    '''Remove the specified ActiveConnection.

    You have to specify the device to remove the connection from, and the
    path of the ActiveConnection.

    Please note that this does not set any global properties.
    '''
    self.SetDeviceDisconnected(dev_path)

    active_connections = self.Get(MAIN_IFACE, 'ActiveConnections')

    if active_connection_path not in active_connections:
        return

    active_connections.remove(dbus.ObjectPath(active_connection_path))
    self.SetProperty(
        MAIN_OBJ, MAIN_IFACE, 'ActiveConnections', active_connections
    )

    self.RemoveObject(active_connection_path)


@dbus.service.method(SETTINGS_IFACE,
                     in_signature='a{sa{sv}}', out_signature='o')
def SettingsAddConnection(self, connection_settings):
    '''Add a connection.

    connection_settings is a String String Variant Map Map. See
    https://developer.gnome.org/NetworkManager/0.9/spec.html
        #type-String_String_Variant_Map_Map

    If you omit uuid, this method adds one for you.
    '''

    if 'uuid' not in connection_settings['connection']:
        connection_settings['connection']['uuid'] = str(uuid.uuid4())

    NM = dbusmock.get_object(MAIN_OBJ)
    settings_obj = dbusmock.get_object(SETTINGS_OBJ)
    main_connections = settings_obj.ListConnections()

    # Mimic how NM names connections
    connection_name = str(len(main_connections))

    connection_path = SETTINGS_OBJ + '/' + connection_name

    if connection_path in main_connections:
        raise dbus.exceptions.DBusException(
            'Connection %s already exists' % connection_path,
            name=MAIN_IFACE + '.AlreadyExists',)

    self.AddObject(
        connection_path,
        CSETTINGS_IFACE,
        {
            'Settings': dbus.Dictionary(
                connection_settings, signature='sa{sv}'
            ),
            'Secrets': dbus.Dictionary({}, signature='sa{sv}'),
        },
        [
            ('Delete', '', '', 'self.ConnectionDelete("%s")' %
                connection_path),
            (
                'GetSettings', '', 'a{sa{sv}}',
                "ret = self.Get('%s', 'Settings')" % CSETTINGS_IFACE
            ),
            (
                'GetSecrets', 's', 'a{sa{sv}}',
                "ret = self.Get('%s', 'Secrets')" % CSETTINGS_IFACE
            ),
            (
                'Update', 'a{sa{sv}}', '',
                'self.ConnectionUpdate("%s", args[0])' % connection_path
            ),
        ]
    )

    main_connections.append(connection_path)
    settings_obj.Set(SETTINGS_IFACE, 'Connections', main_connections)

    settings_obj.EmitSignal(
        SETTINGS_IFACE, 'NewConnection', 'o', [connection_path]
    )

    auto_connect = False
    if 'autoconnect' in connection_settings['connection']:
        auto_connect = connection_settings['connection']['autoconnect']

    if auto_connect:
        dev = None
        devices = NM.GetDevices()

        # Grab the first device.
        if len(devices) > 0:
            dev = devices[0]

        if dev:
            activate_connection(NM, connection_path, dev, connection_path)

    return connection_path


@dbus.service.method(CSETTINGS_IFACE,
                     in_signature='oa{sa{sv}}', out_signature='')
def ConnectionUpdate(self, connection_path, settings):
    '''Update settings on a connection.

    settings is a String String Variant Map Map. See
    https://developer.gnome.org/NetworkManager/0.9/spec.html
        #type-String_String_Variant_Map_Map
    '''
    NM = dbusmock.get_object(MAIN_OBJ)
    settings_obj = dbusmock.get_object(SETTINGS_OBJ)
    conn_obj = dbusmock.get_object(connection_path)

    main_connections = settings_obj.ListConnections()

    if connection_path not in main_connections:
        raise dbus.exceptions.DBusException(
            'Connection %s does not exist' % connection_path,
            name=MAIN_IFACE + '.DoesNotExist',)

    conn_settings = conn_obj.Get(CSETTINGS_IFACE, 'Settings')
    changed_settings = {}
    for key, value in settings.items():
        for k, v in value.items():
            changed_settings[k] = v

            if key not in conn_settings:
                conn_settings[key] = dbus.Dictionary({}, signature='sv')

            conn_settings[key][k] = v

    conn_obj.Set(CSETTINGS_IFACE, 'Settings', conn_settings)

    settings_obj.EmitSignal(
        CSETTINGS_IFACE, 'PropertiesChanged', 'a{sv}', [changed_settings]
    )
    settings_obj.EmitSignal(CSETTINGS_IFACE, 'Updated', '', [])

    auto_connect = False
    if 'autoconnect' in settings['connection']:
        auto_connect = settings['connection']['autoconnect']

    if auto_connect:
        dev = None
        devices = NM.GetDevices()

        # Grab the first device.
        if len(devices) > 0:
            dev = devices[0]

        if dev:
            activate_connection(NM, connection_path, dev, connection_path)

    return connection_path


@dbus.service.method(CSETTINGS_IFACE,
                     in_signature='o', out_signature='')
def ConnectionDelete(self, connection_path):
    '''Deletes a connection.

    This also
        * removes the deleted connection from any device,
        * removes any active connection(s) it might be associated with,
        * removes it from the Settings interface,
        * as well as deletes the object from the mock.

    Note: If this was the only active connection, we change the global
    connection state.
    '''
    NM = dbusmock.get_object(MAIN_OBJ)
    settings_obj = dbusmock.get_object(SETTINGS_OBJ)

    # Find the associated active connection(s).
    active_connections = NM.Get(MAIN_IFACE, 'ActiveConnections')
    associated_active_connections = []
    for ac in active_connections:
        ac_obj = dbusmock.get_object(ac)
        ac_con = ac_obj.Get(ACTIVE_CONNECTION_IFACE, 'Connection')
        if ac_con == connection_path:
            associated_active_connections.append(ac)

    # We found that the connection we are deleting are associated to all
    # active connections and subsequently set the global state to
    # disconnected.
    if len(active_connections) == len(associated_active_connections):
        self.SetGlobalConnectionState(NMState.NM_STATE_DISCONNECTED)

    # Remove the connection from all associated devices.
    # We also remove all associated active connections.
    for dev_path in NM.GetDevices():
        dev_obj = dbusmock.get_object(dev_path)
        connections = dev_obj.Get(DEVICE_IFACE, 'AvailableConnections')

        for ac in associated_active_connections:
            NM.RemoveActiveConnection(dev_path, ac)

        if connection_path not in connections:
            continue

        connections.remove(dbus.ObjectPath(connection_path))
        dev_obj.Set(DEVICE_IFACE, 'AvailableConnections', connections)

    # Remove the connection from the settings interface
    main_connections = settings_obj.ListConnections()
    if connection_path not in main_connections:
        return
    main_connections.remove(connection_path)
    settings_obj.Set(SETTINGS_IFACE, 'Connections', main_connections)
    settings_obj.EmitSignal(
        SETTINGS_IFACE, 'ConnectionRemoved', 'o', [connection_path]
    )

    # Remove the connection from the mock
    connection_obj = dbusmock.get_object(connection_path)
    connection_obj.EmitSignal(CSETTINGS_IFACE, 'Removed', '', [])
    self.RemoveObject(connection_path)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from __future__ import absolute_import

from autopilot.matchers import Eventually
from testtools.matchers import Contains, Equals

from time import sleep

from lomiri_system_settings.tests import (
    PhoneOfonoBaseTestCase,
    CALL_FWD_IFACE,
    CALL_SETTINGS_IFACE,
    SIM_IFACE,
    NETREG_IFACE
)


class PhoneTestCase(PhoneOfonoBaseTestCase):

    def test_call_fwd_unconditional(self):
        # Set busy so we can assert that busy is eventually unset.
        call_fwd_page = self.phone_page.set_forward_on_busy('41444424')

        call_fwd_page = self.phone_page.set_forward_unconditionally('41444424')

        # Check that the forward has been set
        self.assertThat(
            call_fwd_page.get_forward_unconditionally,
            Eventually(Equals('41444424')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceUnconditional')),
            Eventually(Contains('41444424')))

        # Check that all other forward methods are disabled and unset.
        for method in ['fwdBusy', 'fwdLost', 'fwdUnreachable']:
            method_obj = call_fwd_page.wait_select_single(objectName=method)
            self.assertFalse(method_obj.enabled)

        self.assertEqual('', call_fwd_page.get_forward_on_busy())
        self.assertEqual('', call_fwd_page.get_forward_when_no_answer())
        self.assertEqual('', call_fwd_page.get_forward_when_unreachable())

        call_fwd_page.unset_forward_unconditionally()

        # Check that the forward has been unset
        self.assertThat(
            call_fwd_page.get_forward_unconditionally,
            Eventually(Equals('')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceUnconditional')),
            Eventually(Contains('')))

    def test_call_fwd_on_busy(self):
        call_fwd_page = self.phone_page.set_forward_on_busy('41444424')

        # Check that the forward has been set
        self.assertThat(
            call_fwd_page.get_forward_on_busy,
            Eventually(Equals('41444424')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceBusy')),
            Eventually(Contains('41444424')))

        call_fwd_page.unset_forward_on_busy()

        # Check that the forward has been unset
        self.assertThat(
            call_fwd_page.get_forward_on_busy,
            Eventually(Equals('')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceBusy')),
            Eventually(Contains('')))

    def test_call_fwd_when_no_answer(self):
        call_fwd_page = self.phone_page.set_forward_when_no_answer('41444424')

        # Check that the forward has been set
        self.assertThat(
            call_fwd_page.get_forward_when_no_answer,
            Eventually(Equals('41444424')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceNoReply')),
            Eventually(Contains('41444424')))

        call_fwd_page.unset_forward_when_no_answer()

        # Check that the forward has been unset
        self.assertThat(
            call_fwd_page.get_forward_when_no_answer,
            Eventually(Equals('')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceNoReply')),
            Eventually(Contains('')))

    def test_call_fwd_when_unreachable(self):
        call_fwd_page = self.phone_page.set_forward_when_unreachable(
            '41444424'
        )

        # Check that the forward has been set
        self.assertThat(
            call_fwd_page.get_forward_when_unreachable,
            Eventually(Equals('41444424')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceNotReachable')),
            Eventually(Contains('41444424')))

        call_fwd_page.unset_forward_when_unreachable()

        # Check that the forward has been unset
        self.assertThat(
            call_fwd_page.get_forward_when_unreachable,
            Eventually(Equals('')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceNotReachable')),
            Eventually(Contains('')))

    def test_call_waiting(self):
        call_wait = self.phone_page._enter_call_waiting()
        # we have to help our test here, this normally takes quite a while
        self.modem_0.EmitSignal(
            CALL_SETTINGS_IFACE, 'PropertyChanged', 'sv',
            ['VoiceCallWaiting', 'disabled'])
        call_wait.enable_call_waiting()

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_SETTINGS_IFACE,
                                         'VoiceCallWaiting')),
            Eventually(Contains('enabled')))

    def test_call_waiting_switch_not_attached(self):
        self.phone_page._enter_call_waiting()
        self.modem_0.EmitSignal(
            NETREG_IFACE, 'PropertyChanged', 'sv',
            ['Status', 'unregistered'])
        call_wait_switch = self.main_view.wait_select_single(
            objectName='callWaitingSwitch')
        self.assertThat(
            call_wait_switch.enabled,
            Eventually(Equals(False))
        )

    # TODO: Test the Services page itself.
    def test_sim_services(self):
        self.assertThat(
            self.phone_page.get_sim_services_enabled,
            Eventually(Equals(True)))
        self.phone_page.go_to_sim_services()
        self.main_view.go_back()
        self.modem_0.EmitSignal(
            SIM_IFACE, 'PropertyChanged', 'sv',
            ['ServiceNumbers', ''])
        self.assertThat(
            self.phone_page.get_sim_services_enabled,
            Eventually(Equals(False)))


class PhoneDualSimTestCase(PhoneOfonoBaseTestCase):

    use_sims = 2

    def test_call_fwd_unconditional_sim_1(self):
        # Set busy so we can assert that busy is eventually unset.
        call_fwd_page = self.phone_page.set_forward_on_busy('41444424',
                                                            sim=0)
        sleep(1)
        call_fwd_page.set_forward_unconditionally('41444424',)

        # Check that the forward has been set
        self.assertThat(
            call_fwd_page.get_forward_unconditionally,
            Eventually(Equals('41444424')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceUnconditional')),
            Eventually(Contains('41444424')))

        # Check that all other forward methods are disabled and unset.
        for method in ['fwdBusy', 'fwdLost', 'fwdUnreachable']:
            method_obj = call_fwd_page.wait_select_single(objectName=method)
            self.assertFalse(method_obj.enabled)

        self.assertEqual('', call_fwd_page.get_forward_on_busy())
        self.assertEqual('', call_fwd_page.get_forward_when_no_answer())
        self.assertEqual('', call_fwd_page.get_forward_when_unreachable())

        call_fwd_page.unset_forward_unconditionally()

        # Check that the forward has been unset
        self.assertThat(
            call_fwd_page.get_forward_unconditionally,
            Eventually(Equals('')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceUnconditional')),
            Eventually(Contains('')))

    def test_call_fwd_on_busy_sim_1(self):
        call_fwd_page = self.phone_page.set_forward_on_busy('41444424',
                                                            sim=0)

        # Check that the forward has been set
        self.assertThat(
            call_fwd_page.get_forward_on_busy,
            Eventually(Equals('41444424')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceBusy')),
            Eventually(Contains('41444424')))

        call_fwd_page.unset_forward_on_busy()

        # Check that the forward has been unset
        self.assertThat(
            call_fwd_page.get_forward_on_busy,
            Eventually(Equals('')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceBusy')),
            Eventually(Contains('')))

    def test_call_fwd_when_no_answer_sim_1(self):
        call_fwd_page = self.phone_page.set_forward_when_no_answer('41444424',
                                                                   sim=0)

        # Check that the forward has been set
        self.assertThat(
            call_fwd_page.get_forward_when_no_answer,
            Eventually(Equals('41444424')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceNoReply')),
            Eventually(Contains('41444424')))

        call_fwd_page.unset_forward_when_no_answer()

        # Check that the forward has been unset
        self.assertThat(
            call_fwd_page.get_forward_when_no_answer,
            Eventually(Equals('')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceNoReply')),
            Eventually(Contains('')))

    def test_call_fwd_when_unreachable_sim_1(self):
        call_fwd_page = self.phone_page.set_forward_when_unreachable(
            '41444424', sim=0
        )

        # Check that the forward has been set
        self.assertThat(
            call_fwd_page.get_forward_when_unreachable,
            Eventually(Equals('41444424')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceNotReachable')),
            Eventually(Contains('41444424')))

        call_fwd_page.unset_forward_when_unreachable()

        # Check that the forward has been unset
        self.assertThat(
            call_fwd_page.get_forward_when_unreachable,
            Eventually(Equals('')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_FWD_IFACE,
                                         'VoiceNotReachable')),
            Eventually(Contains('')))

    def test_call_fwd_unconditional_sim_2(self):
        # Set busy so we can assert that busy is eventually unset.
        call_fwd_page = self.phone_page.set_forward_on_busy('41444424',
                                                            sim=1)
        sleep(1)
        call_fwd_page.set_forward_unconditionally('41444424',)

        # Check that the forward has been set
        self.assertThat(
            call_fwd_page.get_forward_unconditionally,
            Eventually(Equals('41444424')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_1.Get(CALL_FWD_IFACE,
                                         'VoiceUnconditional')),
            Eventually(Contains('41444424')))

        # Check that all other forward methods are disabled and unset.
        for method in ['fwdBusy', 'fwdLost', 'fwdUnreachable']:
            method_obj = call_fwd_page.wait_select_single(objectName=method)
            self.assertFalse(method_obj.enabled)

        self.assertEqual('', call_fwd_page.get_forward_on_busy())
        self.assertEqual('', call_fwd_page.get_forward_when_no_answer())
        self.assertEqual('', call_fwd_page.get_forward_when_unreachable())

        call_fwd_page.unset_forward_unconditionally()

        # Check that the forward has been unset
        self.assertThat(
            call_fwd_page.get_forward_unconditionally,
            Eventually(Equals('')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_1.Get(CALL_FWD_IFACE,
                                         'VoiceUnconditional')),
            Eventually(Contains('')))

    def test_call_fwd_on_busy_sim_2(self):
        call_fwd_page = self.phone_page.set_forward_on_busy('41444424',
                                                            sim=1)

        # Check that the forward has been set
        self.assertThat(
            call_fwd_page.get_forward_on_busy,
            Eventually(Equals('41444424')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_1.Get(CALL_FWD_IFACE,
                                         'VoiceBusy')),
            Eventually(Contains('41444424')))

        call_fwd_page.unset_forward_on_busy()

        # Check that the forward has been unset
        self.assertThat(
            call_fwd_page.get_forward_on_busy,
            Eventually(Equals('')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_1.Get(CALL_FWD_IFACE,
                                         'VoiceBusy')),
            Eventually(Contains('')))

    def test_call_fwd_when_no_answer_sim_2(self):
        call_fwd_page = self.phone_page.set_forward_when_no_answer('41444424',
                                                                   sim=1)

        # Check that the forward has been set
        self.assertThat(
            call_fwd_page.get_forward_when_no_answer,
            Eventually(Equals('41444424')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_1.Get(CALL_FWD_IFACE,
                                         'VoiceNoReply')),
            Eventually(Contains('41444424')))

        call_fwd_page.unset_forward_when_no_answer()

        # Check that the forward has been unset
        self.assertThat(
            call_fwd_page.get_forward_when_no_answer,
            Eventually(Equals('')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_1.Get(CALL_FWD_IFACE,
                                         'VoiceNoReply')),
            Eventually(Contains('')))

    def test_call_fwd_when_unreachable_sim_2(self):
        call_fwd_page = self.phone_page.set_forward_when_unreachable(
            '41444424', sim=1
        )

        # Check that the forward has been set
        self.assertThat(
            call_fwd_page.get_forward_when_unreachable,
            Eventually(Equals('41444424')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_1.Get(CALL_FWD_IFACE,
                                         'VoiceNotReachable')),
            Eventually(Contains('41444424')))

        call_fwd_page.unset_forward_when_unreachable()

        # Check that the forward has been unset
        self.assertThat(
            call_fwd_page.get_forward_when_unreachable,
            Eventually(Equals('')))

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_1.Get(CALL_FWD_IFACE,
                                         'VoiceNotReachable')),
            Eventually(Contains('')))

    def test_call_waiting_sim_1(self):
        call_wait = self.phone_page._enter_call_waiting(sim=0)
        # we have to help our test here, this normally takes quite a while
        self.modem_0.EmitSignal(
            CALL_SETTINGS_IFACE, 'PropertyChanged', 'sv',
            ['VoiceCallWaiting', 'disabled'])
        call_wait.enable_call_waiting()

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_0.Get(CALL_SETTINGS_IFACE,
                                         'VoiceCallWaiting')),
            Eventually(Contains('enabled')))

    def test_call_waiting_sim_2(self):
        call_wait = self.phone_page._enter_call_waiting(sim=1)
        # we have to help our test here, this normally takes quite a while
        self.modem_1.EmitSignal(
            CALL_SETTINGS_IFACE, 'PropertyChanged', 'sv',
            ['VoiceCallWaiting', 'disabled'])
        call_wait.enable_call_waiting()

        # Check that dbus properties have been updated
        self.assertThat(
            lambda: str(self.modem_1.Get(CALL_SETTINGS_IFACE,
                                         'VoiceCallWaiting')),
            Eventually(Contains('enabled')))

    def test_call_waiting_switch_not_attached_sim_1(self):
        self.phone_page._enter_call_waiting(sim=0)
        self.modem_0.EmitSignal(
            NETREG_IFACE, 'PropertyChanged', 'sv',
            ['Status', 'unregistered'])
        call_wait_switch = self.main_view.wait_select_single(
            objectName='callWaitingSwitch')
        self.assertThat(
            call_wait_switch.enabled,
            Eventually(Equals(False))
        )

    def test_call_waiting_switch_not_attached_sim_2(self):
        self.phone_page._enter_call_waiting(sim=1)
        self.modem_1.EmitSignal(
            NETREG_IFACE, 'PropertyChanged', 'sv',
            ['Status', 'unregistered'])
        call_wait_switch = self.main_view.wait_select_single(
            objectName='callWaitingSwitch')
        self.assertThat(
            call_wait_switch.enabled,
            Eventually(Equals(False))
        )

    # TODO: Test the Services page itself.
    def test_sim_services_sim_1(self):
        self.assertThat(
            lambda: self.phone_page.get_sim_services_enabled(sim=0),
            Eventually(Equals(True)))
        self.phone_page.go_to_sim_services(sim=0)
        self.main_view.go_back()
        self.modem_0.EmitSignal(
            SIM_IFACE, 'PropertyChanged', 'sv',
            ['ServiceNumbers', ''])
        self.assertThat(
            lambda: self.phone_page.get_sim_services_enabled(sim=0),
            Eventually(Equals(False)))

    # TODO: Test the Services page itself.
    def test_sim_services_sim_2(self):
        self.assertThat(
            lambda: self.phone_page.get_sim_services_enabled(sim=1),
            Eventually(Equals(True)))
        self.phone_page.go_to_sim_services(sim=1)
        self.main_view.go_back()
        self.modem_1.EmitSignal(
            SIM_IFACE, 'PropertyChanged', 'sv',
            ['ServiceNumbers', ''])
        self.assertThat(
            lambda: self.phone_page.get_sim_services_enabled(sim=1),
            Eventually(Equals(False)))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from __future__ import absolute_import

from autopilot.matchers import Eventually
from testtools.matchers import Contains, Equals
from lomiri_system_settings.tests import LanguageBaseTestCase
from lomiri_system_settings.utils.i18n import ugettext as _


class LanguageTestCase(LanguageBaseTestCase):
    """Tests for Language Page"""

    def test_language_page_title_is_correct(self):
        """Checks whether Language page is available"""
        self.assertThat(
            self.language_page.title,
            Equals(_('Language & Text')))

    def test_change_language(self):
        current_language = self.language_page.get_current_language()
        target_language = current_language + 1
        self.language_page.change_display_language(target_language)

        self.assertThat(
            lambda:
                self.language_page.get_current_language(),
            Eventually(Equals(target_language)))

        self.addCleanup(
            self.language_page.change_display_language, current_language)

        self.assertThat(
            lambda: str(self.session_mock.GetCalls()),
            Eventually(Contains('Reboot')))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from autopilot.matchers import Eventually
from testtools.matchers import Equals

from lomiri_system_settings.tests import HotspotBaseTestCase

from lomiri_system_settings.tests.connectivity import (
    PRIV_IFACE as CTV_PRIV_IFACE, NETS_IFACE as CTV_NETS_IFACE
)


class HotspotSetupTestCase(HotspotBaseTestCase):

    connectivity_parameters = {
        'HotspotEnabled': False,
        'HotspotStored': False,
        'WifiEnabled': True,
        'HotspotSwitchEnabled': True
    }

    def test_setup(self):
        ssid = 'bar'
        password = 'zomgzomg'
        config = {'ssid': ssid, 'password': password}

        self.hotspot_page.setup_hotspot(config)

        # Assert that the switch is on.
        self.assertTrue(self.hotspot_page.get_hotspot_status())

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotEnabled'),
            Eventually(Equals(True))
        )

        self.assertThat(
            lambda: bytearray(
                self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotSsid')
            ).decode('UTF-8'),
            Eventually(Equals(ssid))
        )

        self.assertThat(
            lambda: self.ctv_private.Get(CTV_PRIV_IFACE, 'HotspotPassword'),
            Eventually(Equals(password))
        )

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotStored'),
            Eventually(Equals(True))
        )

    def test_insecure_setup(self):
        ssid = 'bar'
        auth = 'none'
        config = {'ssid': ssid, 'auth': auth}

        self.hotspot_page.setup_hotspot(config)

        # Assert that the switch is on.
        self.assertTrue(self.hotspot_page.get_hotspot_status())

        self.assertThat(
            lambda: self.ctv_private.Get(CTV_PRIV_IFACE, 'HotspotAuth'),
            Eventually(Equals(auth))
        )

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotStored'),
            Eventually(Equals(True))
        )


class HotspotExistsTestCase(HotspotBaseTestCase):

    connectivity_parameters = {
        'HotspotStored': True,
        'WifiEnabled': True,
        'HotspotSwitchEnabled': True
    }

    def test_enabling(self):
        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotEnabled'),
            Eventually(Equals(False))
        )

        self.hotspot_page.enable_hotspot()

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotEnabled'),
            Eventually(Equals(True))
        )

    def test_changing(self):
        ssid = 'bar'
        config = {'ssid': ssid}
        self.hotspot_page.setup_hotspot(config)

        self.assertThat(
            lambda: bytearray(
                self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotSsid')
            ).decode('UTF-8'),
            Eventually(Equals(ssid))
        )


class HotspotRunningTestCase(HotspotBaseTestCase):

    connectivity_parameters = {
        'HotspotStored': True,
        'HotspotEnabled': True,
        'WifiEnabled': True,
        'HotspotSwitchEnabled': True
    }

    def test_disabling(self):

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotEnabled'),
            Eventually(Equals(True))
        )

        self.hotspot_page.disable_hotspot()

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotEnabled'),
            Eventually(Equals(False))
        )


class HotspotChangeNoWiFiTestCase(HotspotBaseTestCase):

    connectivity_parameters = {
        'HotspotStored': True,
        'HotspotEnabled': False,
        'WifiEnabled': False,
        'HotspotSwitchEnabled': True
    }

    def test_enabling(self):

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotEnabled'),
            Eventually(Equals(False))
        )

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'WifiEnabled'),
            Eventually(Equals(False))
        )

        self.hotspot_page.enable_hotspot()

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotEnabled'),
            Eventually(Equals(True))
        )

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'WifiEnabled'),
            Eventually(Equals(True))
        )


class HotspotSetupNoWiFiTestCase(HotspotBaseTestCase):

    connectivity_parameters = {
        'HotspotStored': False,
        'HotspotEnabled': False,
        'WifiEnabled': False,
        'HotspotSwitchEnabled': True
    }

    def test_setup(self):
        ssid = 'bar'
        config = {'ssid': ssid}

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotStored'),
            Eventually(Equals(False))
        )

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'WifiEnabled'),
            Eventually(Equals(False))
        )

        self.hotspot_page.setup_hotspot(config)

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotStored'),
            Eventually(Equals(True))
        )

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'HotspotEnabled'),
            Eventually(Equals(True))
        )

        self.assertThat(
            lambda: self.ctv_nets.Get(CTV_NETS_IFACE, 'WifiEnabled'),
            Eventually(Equals(True))
        )


class HotspotChangeInFlightModeTestCase(HotspotBaseTestCase):

    connectivity_parameters = {
        'HotspotStored': True,
        'HotspotEnabled': False,
        'WifiEnabled': True,
        'FlightMode': True,
        'HotspotSwitchEnabled': False
    }

    def test_switch_disabled(self):
        self.assertFalse(self.hotspot_page.get_hotspot_possible())


class HotspotSetupInFlightModeTestCase(
        HotspotBaseTestCase):

    connectivity_parameters = {
        'HotspotStored': False,
        'HotspotEnabled': False,
        'WifiEnabled': True,
        'FlightMode': True,
        'HotspotSwitchEnabled': False
    }

    def test_setup_disabled(self):
        setup = self.hotspot_page.select_single(
            objectName='hotspotSetupButton'
        )
        self.assertFalse(setup.enabled)
        self.assertFalse(self.hotspot_page.get_hotspot_possible())



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

import dbusmock
import subprocess
from time import sleep

from autopilot.matchers import Eventually
from testtools.matchers import Equals, GreaterThan

from lomiri_system_settings.tests import LomiriSystemSettingsTestCase
from lomiri_system_settings.utils.i18n import ugettext as _

from lomiriuitoolkit import emulators as toolkit_emulators


class TimeDateTestCase(LomiriSystemSettingsTestCase,
                       dbusmock.DBusTestCase):
    """ Tests for the Time and Date Page """

    @classmethod
    def setUpClass(cls):
        cls.start_system_bus()
        cls.dbus_con = cls.get_dbus(True)

    def tearDown(self):
        self.p_mock.terminate()
        self.p_mock.wait()
        super(TimeDateTestCase, self).tearDown()

    def setUp(self):
        """ Go to Time and Date page """
        (self.p_mock, self.obj_timedate1) = self.spawn_server_template(
            'timedated', {}, stdout=subprocess.PIPE
        )
        self.obj_timedate1.Reset()
        super(TimeDateTestCase, self).setUp()
        self.page = self.main_view.go_to_datetime_page()

    @property
    def tz_page(self):
        return self.main_view.select_single(
            objectName='timeZone'
        )

    @staticmethod
    def wait_select_listview_first(listview):
        ntries = 0
        while True:
            try:
                return listview.select_many(toolkit_emulators.Standard)[0]
            except IndexError as e:
                if ntries < 10:
                    ntries += 1
                    sleep(1)
                else:
                    raise e

    def click_tz_search_field(self):
        self.main_view.pointing_device.click_object(self.tz_page)
        text_field = self.main_view.wait_select_single(
            objectName='selectTimeZoneField'
        )
        self.main_view.pointing_device.move_to_object(
            text_field)
        return text_field

    def search_kb_type(self, kb_type):
        search_field = self.click_tz_search_field()
        search_field.write(kb_type)
        return self.main_view.wait_select_single(
            objectName='locationsListView'
        )

    def test_time_date_page(self):
        """ Checks whether Time and Date page is available """
        self.assertEqual(self.page.title, _('Time and Date'))

    def test_tz_list_initially_empty(self):
        """ Checks that no list is displayed initially """
        self.main_view.scroll_to_and_click(self.tz_page)
        labelVisible = self.main_view.wait_select_single(
            objectName='nothingLabel').visible
        self.assertEqual(labelVisible, True)

    def test_searching_tz(self):
        """ Check that searching for a valid location shows something """
        listview = self.search_kb_type('London, United Kingdom')
        self.assertThat(listview.count, Eventually(GreaterThan(0)))

    def test_searching_tz_not_found(self):
        """ Check that searching for an invalid location shows nothing """
        listview = self.search_kb_type('Oh no you don\'t!')
        self.assertEqual(listview.count, 0)
        labelVisible = self.main_view.select_single(
            objectName='nothingLabel').visible
        self.assertEqual(labelVisible, True)

    def test_manual_tz_selection(self):
        """ Check that manually selecting a timezone sets it properly """
        listview = self.search_kb_type('London, United Kingdom')
        london = TimeDateTestCase.wait_select_listview_first(listview)
        self.main_view.pointing_device.click_object(london)
        self.assertThat(self.tz_page.text, Eventually(Equals('Europe/London')))

    def test_same_tz_selection(self):
        """Check that manually setting a timezone then setting the same one
        doesn't take you back to the index.
        """
        # Set tz to London
        listview = self.search_kb_type('London, United Kingdom')
        london = TimeDateTestCase.wait_select_listview_first(listview)
        self.main_view.pointing_device.click_object(london)
        sleep(2)
        # This is also in Europe/London
        listview = self.search_kb_type('Preston, United Kingdom')
        preston = TimeDateTestCase.wait_select_listview_first(listview)
        self.main_view.pointing_device.click_object(preston)

        # The timer is 1 second, wait and see that we haven't moved pages
        sleep(2)
        header = self.main_view.wait_select_single(
            objectName='MainView_Header'
        )
        self.assertThat(header.title, Eventually(Equals(_('Time zone'))))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from __future__ import absolute_import

from time import sleep

from autopilot.matchers import Eventually
from gi.repository import Gio, GLib
from testtools.matchers import Contains, Equals

from lomiri_system_settings.tests import ResetBaseTestCase
from lomiri_system_settings.utils.i18n import ugettext as _


class ResetTestCase(ResetBaseTestCase):
    """Tests for Reset Page"""

    def set_unity_launcher(self, gsettings, key, value):
        gsettings.set_value(key, value)
        # wait for gsettings
        sleep(1)

    def test_reset_page_title_is_correct(self):
        """Checks whether Reset page is available"""
        self.assertThat(
            self.reset_page.title,
            Equals(_('Reset device')))

    def test_reset_launcher(self):
        gsettings = Gio.Settings.new('com.lomiri.Shell.Launcher')

        favorites = gsettings.get_value('favorites')
        gsettings.set_value(
            'favorites',
            GLib.Variant('as', ['application://nautilus.desktop']))
        self.addCleanup(
            self.set_unity_launcher, gsettings, 'favorites', favorites)

        items = gsettings.get_value('items')
        gsettings.set_value(
            'items', GLib.Variant('as', ['application:///dialer-app.desktop']))
        self.addCleanup(
            self.set_unity_launcher, gsettings, 'items', items)

        self.reset_page.reset_launcher()
        self.assertThat(
            lambda: gsettings.get_value('favorites'),
            Eventually(Equals(gsettings.get_default_value('favorites'))))

        self.assertThat(
            lambda: gsettings.get_value('items'),
            Eventually(Equals(gsettings.get_default_value('items'))))

    def test_factory_reset(self):
        self.reset_page.erase_and_reset_everything()
        self.assertThat(
            lambda: str(self.sys_mock.GetCalls()),
            Eventually(Contains('FactoryReset')))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

import locale
import gettext

APP_NAME = 'lomiri-system-settings'
LOCALE_DIR = '/usr/share/locale/'

lc, encoding = locale.getdefaultlocale()

if not lc:
    lc = 'C'

language = gettext.translation(
    APP_NAME, LOCALE_DIR, languages=[lc], fallback=True
)

# UTF-8
ugettext = language.gettext



'''system image D-BUS mock template'''

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.  See http://www.gnu.org/copyleft/lgpl.html for the full text
# of the license.
import dbus

__author__ = 'Jonas G. Drange'
__email__ = 'jonas.drange@canonical.com'
__copyright__ = '(c) 2015 Canonical Ltd.'
__license__ = 'LGPL 3+'

BUS_NAME = 'com.ubports.SystemImage'
MAIN_IFACE = 'com.ubports.SystemImage'
MAIN_OBJ = '/Service'
SYSTEM_BUS = True


def information(self):
    return dbus.Dictionary({
        'target_build_number': str(self.si_props['target_build_number']),
        'device_name': self.si_props['device'],
        'last_check_date': self.si_props['last_check_date'],
        'version_detail': self.si_props['version_detail'],
        'channel_name': self.si_props['channel'],
        'last_update_date': self.si_props['last_update_date'],
        'current_build_number': str(self.si_props['build_number'])
    }, signature='ss')


def pausedownload(self):
    return self.si_props['reply_on_pause']


def cancelupdate(self):
    return self.si_props['reply_on_cancel']


def getsetting(self, key):
    return str(self.si_props[key])


def setsetting(self, key, value):
    self.si_props[key] = value
    self.EmitSignal(
        MAIN_IFACE, 'SettingChanged', 'ss', [key, value]
    )


def load(mock, parameters):
    global _parameters
    _parameters = parameters

    mock.si_props = {
        'build_number': _parameters.get('build_number', 0),
        'device': _parameters.get('device', ''),
        'auto_download': _parameters.get('auto_download', -1),
        'channel': _parameters.get('channel', ''),
        'last_update_date': _parameters.get('last_update_date', ''),
        'last_check_date': _parameters.get('last_check_date', ''),
        'target_build_number': _parameters.get('target_build_number', -1),
        'target_version_detail': _parameters.get('target_version_detail', ''),
        'version_detail': _parameters.get('version_detail', ''),
        'update_available': _parameters.get('update_available', False),
        'reply_on_pause': _parameters.get('reply_on_pause', ''),
        'reply_on_cancel': _parameters.get('reply_on_cancel', ''),
        'failures_before_warning': _parameters.get('failures_before_warning',
                                                   None),
    }

    mock.information = information
    mock.pausedownload = pausedownload
    mock.cancelupdate = cancelupdate
    mock.getsetting = getsetting
    mock.setsetting = setsetting

    mock.AddMethods(MAIN_IFACE, [
        ('Information', '', 'a{ss}', 'ret = self.information(self)'),
        ('Exit', '', '', ''),
        ('ApplyUpdate', '', '', ''),
        ('CheckForUpdate', '', '', ''),
        ('FactoryReset', '', '', ''),
        ('ProductionReset', '', '', ''),
        ('UpdateDownloaded', '', '', ''),
        ('DownloadUpdate', '', '', ''),
        ('ForceAllowGSMDownload', '', '', ''),
        ('GetSetting', 's', 's', 'ret = self.getsetting(self, args[0])'),
        ('SetSetting', 'ss', '',
         'ret = self.setsetting(self, args[0], args[1])'),
        ('PauseDownload', '', 's', 'ret = self.pausedownload(self)'),
        ('CancelUpdate', '', 's', 'ret = self.cancelupdate(self)'),
    ])



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

""" Tests for Lomiri System Settings """

from __future__ import absolute_import

import dbus
import dbusmock
import os
import random
import subprocess
import lomiriuitoolkit

from datetime import datetime
from time import sleep

from autopilot import platform
from autopilot.matchers import Eventually
from dbusmock.templates.networkmanager import (InfrastructureMode,
                                               NM80211ApSecurityFlags)
from fixtures import EnvironmentVariable
from gi.repository import UPowerGlib
from testtools.matchers import Equals, NotEquals, GreaterThan
from lomiri_system_settings.utils.mock_update_click_server import (
    Manager
)
from lomiri_system_settings.tests.connectivity import (
    PRIV_OBJ as CTV_PRIV_OBJ, NETS_OBJ as CTV_NETS_OBJ,
    MAIN_IFACE as CTV_IFACE
)
from lomiriuitoolkit._custom_proxy_objects._common import (
    is_process_running, _start_process, _stop_process)


ACCOUNTS_IFACE = 'org.freedesktop.Accounts'
ACCOUNTS_USER_IFACE = 'org.freedesktop.Accounts.User'
ACCOUNTS_OBJ = '/org/freedesktop/Accounts'
ACCOUNTS_SERVICE = 'com.canonical.unity.AccountsService'
ACCOUNTS_SOUND_IFACE = 'com.lomiri.touch.AccountsService.Sound'
ACCOUNTS_PHONE_IFACE = 'com.lomiri.touch.AccountsService.Phone'
INDICATOR_NETWORK = 'indicator-network'
ISOUND_SERVICE = 'org.ayatana.indicator.sound'
ISOUND_ACTION_PATH = '/org/ayatana/indicator/sound'
GTK_ACTIONS_IFACE = 'org.gtk.Actions'
MODEM_IFACE = 'org.ofono.Modem'
CONNMAN_IFACE = 'org.ofono.ConnectionManager'
RDO_IFACE = 'org.ofono.RadioSettings'
SIM_IFACE = 'org.ofono.SimManager'
NETREG_IFACE = 'org.ofono.NetworkRegistration'
NETOP_IFACE = 'org.ofono.NetworkOperator'
CALL_FWD_IFACE = 'org.ofono.CallForwarding'
CALL_SETTINGS_IFACE = 'org.ofono.CallSettings'
SYSTEM_IFACE = 'com.ubports.SystemImage'
SYSTEM_SERVICE_OBJ = '/Service'
LM_SERVICE = 'org.freedesktop.login1'
LM_PATH = '/org/freedesktop/login1'
LM_IFACE = 'org.freedesktop.login1.Manager'
NM_SERVICE = 'org.freedesktop.NetworkManager'
NM_PATH = '/org/freedesktop/NetworkManager'
NM_IFACE = 'org.freedesktop.NetworkManager'
NM_AC_CON_IFACE = 'org.freedesktop.NetworkManager.Connection.Active'
CON_SERVICE = 'com.lomiri.connectivity1'
CON_PATH = '/com/lomiri/connectivity1/Private'
CON_IFACE = 'com.lomiri.connectivity1.Private'
UPOWER_VERSION = str(UPowerGlib.MAJOR_VERSION)
UPOWER_VERSION += '.' + str(UPowerGlib.MINOR_VERSION)


class LomiriSystemSettingsTestCase(
        lomiriuitoolkit.base.LomiriUIToolkitAppTestCase):

    """Base class for Lomiri System Settings."""

    BINARY = 'lomiri-system-settings'
    DESKTOP_FILE = '/usr/share/applications/lomiri-system-settings.desktop'

    def setUp(self, panel=None):
        super(LomiriSystemSettingsTestCase, self).setUp()
        self.system_settings = self.launch(panel)
        self.main_view = self.system_settings.main_view
        self.assertThat(
            self.main_view.visible,
            Eventually(Equals(True)))

    def set_orientation(self, gsettings, value):
        gsettings.set_value('rotation-lock', value)
        self.assertThat(
            lambda: gsettings.get_value('rotation-lock').get_boolean(),
            Eventually(Equals(value.get_boolean())))

    def launch(self, panel=None):
        """Launch system settings application

        :param testobj: An AutopilotTestCase object, needed to call
        testobj.launch_test_application()

        :param panel: Launch to a specific panel. Default None.

        :returns: A proxy object that represents the application. Introspection
        data is retrievable via this object.
        """
        params = [self.BINARY]
        if platform.model() != 'Desktop':
            params.append('--desktop_file_hint={}'.format(self.DESKTOP_FILE))

        # Launch to a specific panel
        if panel is not None:
            params.append(panel)

        return self.launch_test_application(
            *params,
            app_type='qt',
            emulator_base=lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase,
            capture_output=True)

    @classmethod
    def tearDownClass(cls):
        if dbusmock.DBusTestCase.system_bus_pid is not None:
            cls.stop_dbus(dbusmock.DBusTestCase.system_bus_pid)
            del os.environ['DBUS_SYSTEM_BUS_ADDRESS']
            dbusmock.DBusTestCase.system_bus_pid = None
        if dbusmock.DBusTestCase.session_bus_pid is not None:
            cls.stop_dbus(dbusmock.DBusTestCase.session_bus_pid)
            del os.environ['DBUS_SESSION_BUS_ADDRESS']
            dbusmock.DBusTestCase.session_bus_pid = None
        super(LomiriSystemSettingsTestCase, cls).tearDownClass()


class LomiriSystemSettingsUpowerTestCase(LomiriSystemSettingsTestCase,
                                         dbusmock.DBusTestCase):
    """Base class for battery tests that mocks Upower"""

    @classmethod
    def setUpClass(cls):
        cls.start_system_bus()
        cls.dbus_con = cls.get_dbus(True)

    def setUp(self, panel=None):
        # Add a mock Upower environment so we get consistent results
        (self.p_mock, self.obj_upower) = self.spawn_server_template(
            'upower',
            {'OnBattery': True, 'DaemonVersion': UPOWER_VERSION},
            stdout=subprocess.PIPE)
        self.dbusmock = dbus.Interface(self.obj_upower, dbusmock.MOCK_IFACE)
        self.obj_upower.Reset()
        super(LomiriSystemSettingsUpowerTestCase, self).setUp()

    def add_mock_battery(self):
        """ Make sure we have a battery """
        self.dbusmock.AddDischargingBattery(
            'mock_BATTERY', 'Battery', 50.0, 10
        )

    def tearDown(self):
        self.p_mock.terminate()
        self.p_mock.wait()
        super(LomiriSystemSettingsUpowerTestCase, self).tearDown()


class LomiriSystemSettingsBatteryTestCase(LomiriSystemSettingsUpowerTestCase):
    """ Base class for tests which rely on the presence of a battery """

    def setUp(self):
        super(LomiriSystemSettingsBatteryTestCase, self).setUp()
        self.add_mock_battery()


class LomiriSystemSettingsHotspotTestCase(LomiriSystemSettingsTestCase,
                                          dbusmock.DBusTestCase):
    """Base class for tests that tests the hotspot functionality."""
    connectivity_parameters = {}
    indicatornetwork_parameters = {}
    systemimage_parameters = {'device': 'ideal'}

    @classmethod
    def setUpClass(cls):
        cls.session_con = cls.get_dbus(False)

        cls.start_system_bus()

        si_tmpl = os.path.join(os.path.dirname(__file__), 'systemimage.py')
        (cls.si_mock, cls.si_obj) = cls.spawn_server_template(
            si_tmpl, parameters=cls.systemimage_parameters,
            stdout=subprocess.PIPE)

        super(LomiriSystemSettingsHotspotTestCase, cls).setUpClass()

    def setUp(self):
        if is_process_running(INDICATOR_NETWORK):
            _stop_process(INDICATOR_NETWORK)
            self.addCleanup(_start_process, INDICATOR_NETWORK)

        ctv_tmpl = os.path.join(os.path.dirname(__file__), 'connectivity.py')
        (self.ctv_mock, self.obj_ctv) = self.spawn_server_template(
            ctv_tmpl, parameters=self.connectivity_parameters,
            stdout=subprocess.PIPE)

        self.ctv_private = dbus.Interface(
            self.session_con.get_object(CTV_IFACE, CTV_PRIV_OBJ),
            'org.freedesktop.DBus.Properties')

        self.ctv_nets = dbus.Interface(
            self.session_con.get_object(CTV_IFACE, CTV_NETS_OBJ),
            'org.freedesktop.DBus.Properties')

        inetwork = os.path.join(
            os.path.dirname(__file__), 'indicatornetwork.py'
        )
        (self.inetwork_mock, self.obj_inetwork) = self.spawn_server_template(
            inetwork, parameters=self.indicatornetwork_parameters,
            stdout=subprocess.PIPE)

        # Required since this test needs to dismiss the OSK.
        self.useFixture(EnvironmentVariable("UITK_USE_MALIIT", "1"))

        super(LomiriSystemSettingsHotspotTestCase, self).setUp()

    def tearDown(self):
        self.ctv_mock.terminate()
        self.ctv_mock.wait()
        self.inetwork_mock.terminate()
        self.inetwork_mock.wait()
        super(LomiriSystemSettingsHotspotTestCase, self).tearDown()

    @classmethod
    def tearDownClass(cls):
        cls.si_mock.terminate()
        cls.si_mock.wait()
        if dbusmock.DBusTestCase.system_bus_pid is not None:
            cls.stop_dbus(dbusmock.DBusTestCase.system_bus_pid)
            del os.environ['DBUS_SYSTEM_BUS_ADDRESS']
            dbusmock.DBusTestCase.system_bus_pid = None
        if dbusmock.DBusTestCase.session_bus_pid is not None:
            cls.stop_dbus(dbusmock.DBusTestCase.session_bus_pid)
            del os.environ['DBUS_SESSION_BUS_ADDRESS']
            dbusmock.DBusTestCase.session_bus_pid = None
        super(LomiriSystemSettingsHotspotTestCase, cls).tearDownClass()


class LomiriSystemSettingsOfonoTestCase(LomiriSystemSettingsTestCase,
                                        dbusmock.DBusTestCase):
    """Class for cellular tests which sets up an Ofono mock """

    use_sims = 1

    @property
    def choose_carrier_page(self):
        """Return carrier selection page"""
        return self.main_view.select_single(
            objectName='chooseCarrierPage'
        )

    @property
    def choose_carriers_page(self):
        """Return carriers selection page"""
        return self.main_view.select_single(
            objectName='chooseCarriersPage'
        )

    # TODO: remove this when it has been fixed in dbusmock
    def get_all_operators(self, name):
        return 'ret = [(m, objects[m].GetAll("org.ofono.NetworkOperator")) ' \
               'for m in objects if "%s/operator/" in m]' % name

    def mock_carriers(self, name):
        self.dbusmock.AddObject(
            '/%s/operator/op2' % name,
            NETOP_IFACE,
            {
                'Name': 'my.cool.telco',
                'Status': 'available',
                'MobileCountryCode': '777',
                'MobileNetworkCode': '22',
                'Technologies': ['gsm'],
            },
            [
                ('GetProperties', '', 'a{sv}',
                    'ret = self.GetAll("org.ofono.NetworkOperator")'),
                ('Register', '', '', ''),
            ]
        )
        # Add a forbidden carrier
        self.dbusmock.AddObject(
            '/%s/operator/op3' % name,
            NETOP_IFACE,
            {
                'Name': 'my.bad.telco',
                'Status': 'forbidden',
                'MobileCountryCode': '777',
                'MobileNetworkCode': '22',
                'Technologies': ['gsm'],
            },
            [
                ('GetProperties', '', 'a{sv}',
                    'ret = self.GetAll("org.ofono.NetworkOperator")'),
                ('Register', '', '', ''),
            ]
        )

    def mock_radio_settings(self, modem, preference='gsm',
                            technologies=['gsm', 'umts', 'lte']):
        modem.AddProperty(
            RDO_IFACE, 'TechnologyPreference', preference)
        modem.AddProperty(
            RDO_IFACE, 'AvailableTechnologies', technologies)
        modem.AddMethods(
            RDO_IFACE,
            [('GetProperties', '', 'a{sv}',
              'ret = self.GetAll("%s")' % RDO_IFACE),
                ('SetProperty', 'sv', '',
                 'self.Set("IFACE", args[0], args[1]); '
                 'self.EmitSignal("IFACE",\
                 "PropertyChanged", "sv", [args[0], args[1]])'
                    .replace('IFACE', RDO_IFACE)), ])

    def mock_call_forwarding(self, modem):
        modem.AddProperties(CALL_FWD_IFACE,
                            {
                                'VoiceUnconditional': '',
                                'VoiceBusy': '',
                                'VoiceNoReply': '',
                                'VoiceNotReachable': '',
                            })
        modem.AddMethods(
            CALL_FWD_IFACE,
            [('GetProperties', '', 'a{sv}',
              'ret = self.GetAll("%s")' % CALL_FWD_IFACE),
                ('SetProperty', 'sv', '',
                 'self.Set("IFACE", args[0], args[1]); '
                 'self.EmitSignal("IFACE",\
                 "PropertyChanged", "sv", [args[0], args[1]])'
                    .replace('IFACE', CALL_FWD_IFACE)), ])

    def mock_call_settings(self, modem):
        modem.AddProperty(
            CALL_SETTINGS_IFACE, 'VoiceCallWaiting', 'disabled')
        modem.AddMethods(
            CALL_SETTINGS_IFACE,
            [('GetProperties', '', 'a{sv}',
              'ret = self.GetAll("%s")' % CALL_SETTINGS_IFACE),
                ('SetProperty', 'sv', '',
                 'self.Set("IFACE", args[0], args[1]); '
                 'self.EmitSignal("IFACE",\
                 "PropertyChanged", "sv", [args[0], args[1]])'
                    .replace('IFACE', CALL_SETTINGS_IFACE)), ])

    def add_sim1(self):
        # create modem_0 proxy
        self.modem_0 = self.dbus_con.get_object('org.ofono', '/ril_0')

        self.mock_carriers('ril_0')
        self.mock_radio_settings(self.modem_0)
        self.mock_call_forwarding(self.modem_0)
        self.mock_call_settings(self.modem_0)

        self.modem_0.AddMethods('org.ofono.NetworkRegistration', [
            ('GetProperties', '', 'a{sv}',
                'ret = self.GetAll("org.ofono.NetworkRegistration")'),
            ('Register', '', '', ''),
            ('GetOperators', '', 'a(oa{sv})', self.get_all_operators('ril_0')),
            ('Scan', '', 'a(oa{sv})', self.get_all_operators('ril_0')),
        ])
        self.modem_0.connMan = dbus.Interface(self.dbus_con.get_object(
                                              'org.ofono', '/ril_0'),
                                              'org.ofono.ConnectionManager')

    def add_sim2(self):
        '''Mock two modems/sims for the dual sim story'''
        second_modem = 'ril_1'

        self.dbusmock.AddModem(second_modem, {'Powered': True})
        self.modem_1 = self.dbus_con.get_object(
            'org.ofono', '/%s' % second_modem)

        self.modem_1.AddMethods(NETREG_IFACE, [
            ('GetProperties', '', 'a{sv}',
                'ret = self.GetAll("org.ofono.NetworkRegistration")'),
            ('Register', '', '', ''),
            ('GetOperators', '', 'a(oa{sv})',
                self.get_all_operators(second_modem)),
            ('Scan', '', 'a(oa{sv})',
                self.get_all_operators(second_modem)),
        ])
        self.mock_carriers(second_modem)
        self.mock_radio_settings(self.modem_1, technologies=['gsm'])
        self.mock_call_forwarding(self.modem_1)
        self.mock_call_settings(self.modem_1)

        self.modem_1.Set(
            SIM_IFACE,
            'SubscriberNumbers', ['08123', '938762783']
        )

        self.modem_1.connMan = dbus.Interface(self.dbus_con.get_object(
                                              'org.ofono', '/' + second_modem),
                                              'org.ofono.ConnectionManager')

    @classmethod
    def setUpClass(cls):
        cls.start_system_bus()
        cls.dbus_con = cls.get_dbus(True)
        super(LomiriSystemSettingsOfonoTestCase, cls).setUpClass()

    def tearDown(self):
        self.p_mock.terminate()
        self.p_mock.wait()
        super(LomiriSystemSettingsOfonoTestCase, self).tearDown()

    def setUp(self, panel=None):
        template = os.path.join(os.path.dirname(__file__), 'ofono.py')
        # Add a mock Ofono environment so we get consistent results
        (self.p_mock, self.obj_ofono) = self.spawn_server_template(
            template, stdout=subprocess.PIPE)
        self.dbusmock = dbus.Interface(self.obj_ofono, dbusmock.MOCK_IFACE)
        self.obj_ofono.Reset()

        self.add_sim1()
        if self.use_sims == 2:
            self.add_sim2()

        super(LomiriSystemSettingsOfonoTestCase, self).setUp()

    def get_default_sim_for_calls_selector(self, text):
        return self.cellular_page.select_single(
            objectName="defaultForCalls" + text
        )

    def get_default_sim_for_messages_selector(self, text):
        return self.cellular_page.select_single(
            objectName="defaultForMessages" + text
        )


class CellularBaseTestCase(LomiriSystemSettingsOfonoTestCase):

    connectivity_parameters = {
        'Status': 'online'
    }

    def setUp(self):
        """ Go to Cellular page """

        self.session_con = self.get_dbus(False)

        if is_process_running(INDICATOR_NETWORK):
            _stop_process(INDICATOR_NETWORK)
            self.addCleanup(_start_process, INDICATOR_NETWORK)

        ctv_tmpl = os.path.join(os.path.dirname(__file__), 'connectivity.py')
        (self.ctv_mock, self.obj_ctv) = self.spawn_server_template(
            ctv_tmpl, parameters=self.connectivity_parameters,
            stdout=subprocess.PIPE)

        sleep(1)

        self.ctv_private = dbus.Interface(
            self.session_con.get_object(CTV_IFACE, CTV_PRIV_OBJ),
            'org.freedesktop.DBus.Properties')

        sim = self.obj_ctv.AddSim("1234567890")
        self.ctv_private.Set(CON_IFACE,
                             'Sims',
                             dbus.Array([sim],
                                        signature='o'))
        modem = self.obj_ctv.AddModem("0987654321", 1, sim)
        self.ctv_private.Set(CON_IFACE,
                             'Modems',
                             dbus.Array([modem],
                                        signature='o'))

        self.ctv_modem0 = dbus.Interface(
            self.session_con.get_object(CTV_IFACE, modem),
            'org.freedesktop.DBus.Properties')
        self.ctv_sim0 = dbus.Interface(
            self.session_con.get_object(CTV_IFACE, sim),
            'org.freedesktop.DBus.Properties')

        if self.use_sims == 2:
            sim2 = self.obj_ctv.AddSim("2345678901")
            self.ctv_private.Set(CON_IFACE,
                                 'Sims',
                                 dbus.Array([sim, sim2],
                                            signature='o'))
            modem2 = self.obj_ctv.AddModem("1098765432", 2, sim2)
            self.ctv_private.Set(CON_IFACE,
                                 'Modems',
                                 dbus.Array([modem, modem2],
                                            signature='o'))

            self.ctv_modem1 = dbus.Interface(
                self.session_con.get_object(CTV_IFACE, modem2),
                'org.freedesktop.DBus.Properties')
            self.ctv_sim1 = dbus.Interface(
                self.session_con.get_object(CTV_IFACE, sim2),
                'org.freedesktop.DBus.Properties')

        user_obj = '/user/foo'

        self.accts_phone_props = {
            'DefaultSimForCalls': dbus.String("/ril_0", variant_level=1),
            'DefaultSimForMessages': dbus.String("/ril_1", variant_level=1),
            'SimNames': dbus.Dictionary({}, signature='ss', variant_level=1)}

        # start dbus system bus
        self.mock_server = self.spawn_server(ACCOUNTS_IFACE, ACCOUNTS_OBJ,
                                             ACCOUNTS_IFACE, system_bus=True,
                                             stdout=subprocess.PIPE)
        # give it time to ensure the mock is up
        sleep(1)

        self.wait_for_bus_object(ACCOUNTS_IFACE,
                                 ACCOUNTS_OBJ,
                                 system_bus=True)

        self.dbus_mock = dbus.Interface(self.dbus_con.get_object(
                                        ACCOUNTS_IFACE,
                                        ACCOUNTS_OBJ,
                                        ACCOUNTS_IFACE),
                                        dbusmock.MOCK_IFACE)

        # let accountservice find a user object path
        self.dbus_mock.AddMethod(ACCOUNTS_IFACE, 'FindUserById', 'x', 'o',
                                 'ret = "%s"' % user_obj)

        self.dbus_mock.AddProperties(ACCOUNTS_PHONE_IFACE,
                                     self.accts_phone_props)

        # add getter and setter to mock
        self.dbus_mock.AddMethods(
            'org.freedesktop.DBus.Properties',
            [
                ('self.Get',
                 's',
                 'v',
                 'ret = self.accts_phone_props[args[0]]'),
                ('self.Set',
                 'sv',
                 '',
                 'self.accts_phone_props[args[0]] = args[1]')
            ])

        # add user object to mock
        self.dbus_mock.AddObject(
            user_obj, ACCOUNTS_PHONE_IFACE, self.accts_phone_props,
            [
                (
                    'GetDefaultSimForCalls', '', 'v',
                    'ret = self.Get("%s", "DefaultSimForCalls")' %
                    ACCOUNTS_PHONE_IFACE),
                (
                    'GetDefaultSimForMessages', '', 'v',
                    'ret = self.Get("%s", "DefaultSimForMessages")' %
                    ACCOUNTS_PHONE_IFACE),
                (
                    'GetSimNames', '', 'v',
                    'ret = self.Get("%s", "SimNames")' %
                    ACCOUNTS_PHONE_IFACE)
            ])

        self.obj_phone = self.dbus_con.get_object(ACCOUNTS_IFACE, user_obj,
                                                  ACCOUNTS_IFACE)

        super(CellularBaseTestCase, self).setUp()
        self.cellular_page = self.main_view.go_to_cellular_page()

    def add_connection_context(self, modem, **kwargs):
        iface = 'org.ofono.ConnectionContext'
        path = modem.connMan.AddContext(kwargs.get('Type', 'internet'))
        context = dbus.Interface(self.dbus_con.get_object(
                                 'org.ofono', path),
                                 iface)

        for key, value in kwargs.items():
            context.SetProperty(key, value)

    def tearDown(self):
        self.ctv_mock.terminate()
        self.ctv_mock.wait()
        self.mock_server.terminate()
        self.mock_server.wait()
        super(CellularBaseTestCase, self).tearDown()


class HotspotBaseTestCase(LomiriSystemSettingsHotspotTestCase):

    def setUp(self):
        super(HotspotBaseTestCase, self).setUp()
        self.hotspot_page = self.main_view.go_to_hotspot_page()


class BluetoothBaseTestCase(LomiriSystemSettingsTestCase):

    def setUp(self):
        """ Go to Bluetooth page """
        super(BluetoothBaseTestCase, self).setUp()
        self.bluetooth_page = self.main_view.go_to_bluetooth_page()


class PhoneOfonoBaseTestCase(LomiriSystemSettingsOfonoTestCase):
    def setUp(self):
        """ Go to Phone page """
        self.useFixture(EnvironmentVariable("LSS_SHOW_ALL_UI", "1"))
        super(PhoneOfonoBaseTestCase, self).setUp()
        self.phone_page = self.main_view.go_to_phone_page()


class AboutBaseTestCase(LomiriSystemSettingsTestCase):
    def setUp(self):
        """Go to About page."""
        super(AboutBaseTestCase, self).setUp()
        self.about_page = self.main_view.go_to_about_page()


class AboutOfonoBaseTestCase(LomiriSystemSettingsOfonoTestCase):
    def setUp(self):
        """Go to About page."""
        super(AboutOfonoBaseTestCase, self).setUp()
        self.about_page = self.main_view.go_to_about_page()


class AboutSystemImageBaseTestCase(AboutBaseTestCase,
                                   dbusmock.DBusTestCase):
    systemimage_parameters = {
        'last_update_date': datetime.now().replace(microsecond=0).isoformat(),
        'channel': 'daily',
        'build_number': 42,
        'device': ''
    }

    @classmethod
    def setUpClass(cls):
        cls.start_system_bus()
        cls.dbus_con = cls.get_dbus(True)
        si_tmpl = os.path.join(os.path.dirname(__file__), 'systemimage.py')
        (cls.si_mock, cls.si_obj) = cls.spawn_server_template(
            si_tmpl, parameters=cls.systemimage_parameters,
            stdout=subprocess.PIPE)

    @classmethod
    def tearDownClass(cls):
        cls.si_mock.terminate()
        cls.si_mock.wait()
        super(AboutSystemImageBaseTestCase, cls).tearDownClass()


class StorageBaseTestCase(AboutBaseTestCase):

    """Base class for Storage page tests."""

    def setUp(self):
        """Go to Storage Page."""
        super(StorageBaseTestCase, self).setUp()
        self.main_view.click_item('storageItem')
        self.storage_page = self.main_view.select_single(
            objectName='storagePage'
        )
        self.assertThat(self.storage_page.active, Eventually(Equals(True)))

    def assert_space_item(self, object_name, text):
        """ Checks whether an space item exists and returns a value """
        item = self.main_view.storage_page.wait_select_single(
            objectName=object_name
        )
        self.assertThat(item, NotEquals(None))
        label = item.label  # Label
        self.assertThat(label, Equals(text))
        # Get item's label
        size_label = item.select_single(objectName='sizeLabel')
        self.assertThat(size_label, NotEquals(None))
        values = size_label.text.split(' ')  # Format: "00.0 (bytes|MB|GB)"
        self.assertThat(len(values), GreaterThan(1))

    def get_storage_space_used_by_category(self, objectName):
        return self.main_view.wait_select_single(
            'StorageItem', objectName=objectName).value


class LicenseBaseTestCase(AboutBaseTestCase):

    """Base class for Licenses page tests."""

    def setUp(self):
        """Go to License Page."""
        super(LicenseBaseTestCase, self).setUp()
        self.licenses_page = self.about_page.go_to_software_licenses()


class SystemUpdatesBaseTestCase(LomiriSystemSettingsTestCase,
                                dbusmock.DBusTestCase):
    """Base class for SystemUpdates page tests."""

    connectivity_parameters = {
        'Status': 'online'
    }

    click_server_parameters = {
        'start': False
    }

    systemimage_parameters = {}

    @classmethod
    def setUpClass(cls):
        cls.session_con = cls.get_dbus(False)

        cls.start_system_bus()

        si_tmpl = os.path.join(os.path.dirname(__file__), 'systemimage.py')
        (cls.si_mock, cls.si_obj) = cls.spawn_server_template(
            si_tmpl, parameters=cls.systemimage_parameters,
            stdout=subprocess.PIPE)

        super(SystemUpdatesBaseTestCase, cls).setUpClass()

    def setUp(self):
        """Go to SystemUpdates Page."""
        self.clicksrv_manager = None
        if is_process_running(INDICATOR_NETWORK):
            _stop_process(INDICATOR_NETWORK)
            self.addCleanup(_start_process, INDICATOR_NETWORK)

        ctv_tmpl = os.path.join(os.path.dirname(__file__), 'connectivity.py')
        (self.ctv_mock, self.obj_ctv) = self.spawn_server_template(
            ctv_tmpl, parameters=self.connectivity_parameters,
            stdout=subprocess.PIPE)

        self.ctv_nets = dbus.Interface(
            self.session_con.get_object(CTV_IFACE, CTV_NETS_OBJ),
            'org.freedesktop.DBus.Properties')

        if self.click_server_parameters['start']:
            self.clicksrv_manager = Manager(
                responses=self.click_server_parameters.get('responses', None)
            )
            self.clicksrv_manager.start()

        super(SystemUpdatesBaseTestCase, self).setUp()
        self.main_view.click_item('entryComponent-system-update')

    def tearDown(self):
        self.ctv_mock.terminate()
        self.ctv_mock.wait()
        if self.clicksrv_manager and self.clicksrv_manager.is_running():
            self.clicksrv_manager.stop()
        super(SystemUpdatesBaseTestCase, self).tearDown()

    @classmethod
    def tearDownClass(cls):
        cls.si_mock.terminate()
        cls.si_mock.wait()
        if dbusmock.DBusTestCase.system_bus_pid is not None:
            cls.stop_dbus(dbusmock.DBusTestCase.system_bus_pid)
            del os.environ['DBUS_SYSTEM_BUS_ADDRESS']
            dbusmock.DBusTestCase.system_bus_pid = None
        if dbusmock.DBusTestCase.session_bus_pid is not None:
            cls.stop_dbus(dbusmock.DBusTestCase.session_bus_pid)
            del os.environ['DBUS_SESSION_BUS_ADDRESS']
            dbusmock.DBusTestCase.session_bus_pid = None
        super(SystemUpdatesBaseTestCase, cls).tearDownClass()


class BackgroundBaseTestCase(
        LomiriSystemSettingsTestCase,
        dbusmock.DBusTestCase):
    """ Base class for Background tests """

    @classmethod
    def setUpClass(klass):
        klass.start_system_bus()
        klass.dbus_con = klass.get_dbus(True)

    # TODO: create dbusmock template
    def setUp(self):
        """Mock account service dbus, go to background page"""

        # mock ubuntu art directory using a local path
        art_dir = '%s/../background_images/' % (
            os.path.dirname(os.path.realpath(__file__)))
        user_obj = '/user/foo'

        self.user_props = {
            'BackgroundFile': dbus.String(
                '%slaunchpad.jpg' % art_dir, variant_level=1)
        }

        # start dbus system bus
        self.mock_server = self.spawn_server(ACCOUNTS_IFACE, ACCOUNTS_OBJ,
                                             ACCOUNTS_IFACE, system_bus=True,
                                             stdout=subprocess.PIPE)

        sleep(2)

        # create account proxy
        self.acc_proxy = dbus.Interface(self.dbus_con.get_object(
            ACCOUNTS_IFACE, ACCOUNTS_OBJ), dbusmock.MOCK_IFACE)

        # let accountservice find a user object path
        self.acc_proxy.AddMethod(ACCOUNTS_IFACE, 'FindUserById', 'x', 'o',
                                 'ret = "%s"' % user_obj)

        # add getter and setter to mock
        self.acc_proxy.AddMethods(
            'org.freedesktop.DBus.Properties',
            [('Get', 's', 'v', 'ret = self.user_props[args[0]]'),
                ('Set', 'sv', '', 'self.user_props[args[0]] = args[1]')])

        # add user object to mock
        self.acc_proxy.AddObject(
            user_obj, ACCOUNTS_USER_IFACE, self.user_props,
            [
                (
                    'SetBackgroundFile', 'v', '',
                    'self.Set("%s", "BackgroundFile", args[0]);' %
                    ACCOUNTS_USER_IFACE),
                (
                    'GetBackgroundFile', '', 'v',
                    'ret = self.Get("%s", "BackgroundFile")' %
                    ACCOUNTS_USER_IFACE)
            ])

        # create user proxy
        self.user_proxy = dbus.Interface(self.dbus_con.get_object(
            ACCOUNTS_IFACE, user_obj),
            ACCOUNTS_USER_IFACE)

        # patch env variable
        self.useFixture(EnvironmentVariable(
            'SYSTEM_SETTINGS_SYSTEM_ART_DIR', art_dir))

        super(BackgroundBaseTestCase, self).setUp('background')
        self.assertThat(self.main_view.background_page.active,
                        Eventually(Equals(True)))

    def tearDown(self):
        self.mock_server.terminate()
        self.mock_server.wait()
        super(BackgroundBaseTestCase, self).tearDown()


class SoundBaseTestCase(
        LomiriSystemSettingsTestCase,
        dbusmock.DBusTestCase):
    """ Base class for sound settings tests"""

    @classmethod
    def setUpClass(klass):
        klass.start_system_bus()
        klass.dbus_con = klass.get_dbus(True)
        klass.dbus_con_session = klass.get_dbus(False)

    def setUp(self, panel='sound'):
        # TODO only do this if the sound indicator is running.
        # --elopio - 2015-01-08
        self.stop_sound_indicator()
        self.addCleanup(self.start_sound_indicator)

        user_obj = '/user/foo'

        self.accts_snd_props = {
            'IncomingCallVibrate': dbus.Boolean(False, variant_level=1),
            'IncomingCallVibrateSilentMode': dbus.Boolean(False,
                                                          variant_level=1),
            'IncomingMessageVibrate': dbus.Boolean(False,
                                                   variant_level=1),
            'IncomingMessageVibrateSilentMode': dbus.Boolean(False,
                                                             variant_level=1),
            'DialpadSoundsEnabled': dbus.Boolean(True,
                                                 variant_level=1)}

        # start dbus system bus
        self.mock_server = self.spawn_server(ACCOUNTS_IFACE, ACCOUNTS_OBJ,
                                             ACCOUNTS_IFACE, system_bus=True,
                                             stdout=subprocess.PIPE)

        # start isound
        self.mock_isound = self.spawn_server(ISOUND_SERVICE,
                                             ISOUND_ACTION_PATH,
                                             GTK_ACTIONS_IFACE,
                                             stdout=subprocess.PIPE)

        self.wait_for_bus_object(ACCOUNTS_IFACE,
                                 ACCOUNTS_OBJ,
                                 system_bus=True)

        self.dbus_mock = dbus.Interface(self.dbus_con.get_object(
                                        ACCOUNTS_IFACE,
                                        ACCOUNTS_OBJ,
                                        ACCOUNTS_IFACE),
                                        dbusmock.MOCK_IFACE)

        self.wait_for_bus_object(ISOUND_SERVICE,
                                 ISOUND_ACTION_PATH)

        self.dbus_mock_isound = dbus.Interface(
            self.dbus_con_session.get_object(
                ISOUND_SERVICE,
                ISOUND_ACTION_PATH,
                GTK_ACTIONS_IFACE),
            dbusmock.MOCK_IFACE)

        # let accountservice find a user object path
        self.dbus_mock.AddMethod(ACCOUNTS_IFACE, 'FindUserById', 'x', 'o',
                                 'ret = "%s"' % user_obj)

        self.dbus_mock.AddProperties(ACCOUNTS_SOUND_IFACE,
                                     self.accts_snd_props)

        # add getter and setter to mock
        self.dbus_mock.AddMethods(
            'org.freedesktop.DBus.Properties',
            [
                ('self.Get',
                 's',
                 'v',
                 'ret = self.accts_snd_props[args[0]]'),
                ('self.Set',
                 'sv',
                 '',
                 'self.accts_snd_props[args[0]] = args[1]')
            ])

        # add user object to mock
        self.dbus_mock.AddObject(
            user_obj, ACCOUNTS_SOUND_IFACE, self.accts_snd_props,
            [
                (
                    'GetIncomingCallVibrate', '', 'v',
                    'ret = self.Get("%s", "IncomingCallVibrate")' %
                    ACCOUNTS_SOUND_IFACE),
                (
                    'GetIncomingMessageVibrate', '', 'v',
                    'ret = self.Get("%s", "IncomingMessageVibrate")' %
                    ACCOUNTS_SOUND_IFACE),
                (
                    'GetIncomingCallVibrateSilentMode', '', 'v',
                    'ret = self.Get("%s", "IncomingCallVibrateSilentMode")' %
                    ACCOUNTS_SOUND_IFACE),
                (
                    'GetIncomingMessageVibrateSilentMode', '', 'v',
                    'ret = self.Get("%s", \
                                    "IncomingMessageVibrateSilentMode")' %
                    ACCOUNTS_SOUND_IFACE),
                (
                    'GetDialpadSoundsEnabled', '', 'v',
                    'ret = self.Get("%s", \
                                    "DialpadSoundsEnabled")' %
                    ACCOUNTS_SOUND_IFACE)
            ])

        self.obj_snd = self.dbus_con.get_object(ACCOUNTS_IFACE, user_obj,
                                                ACCOUNTS_IFACE)

        self.dbus_mock_isound.AddMethods(
            GTK_ACTIONS_IFACE,
            [
                (
                    'Activate', 'sava{sv}', '',
                    ''
                ),
                (
                    'Describe', 's', '(bsav)',
                    'if args[0] == "silent-mode":'
                    '    ret = [True, "", [False]]'
                    'if args[0] == "volume":'
                    '    ret = [True, "i", [0.4]]'
                ),
                (
                    'DescribeAll', 's', 'a{s(bsav)}',
                    'ret = {'
                    '"silent-mode": [True, "", [False]],'
                    '"volume": [True, "i", [0.4]]}'
                ),
                (
                    'List', '', 'as',
                    'ret = ["silent-mode", "volume"]'
                ),
                (
                    'SetState', 'sva{sv}', '',
                    ''
                )
            ])

        super(SoundBaseTestCase, self).setUp(panel)

        if panel == 'sound':
            self.sound_page = self.main_view.select_single(
                objectName='soundPage'
            )
            self.assertThat(self.sound_page.active,
                            Eventually(Equals(True)))

    def tearDown(self):
        self.mock_server.terminate()
        self.mock_server.wait()
        self.mock_isound.terminate()
        self.mock_isound.wait()
        super(SoundBaseTestCase, self).tearDown()

    def start_sound_indicator(self):
        subprocess.call(['initctl', 'start', 'indicator-sound'])

    def stop_sound_indicator(self):
        subprocess.call(['initctl', 'stop', 'indicator-sound'])


class ResetBaseTestCase(LomiriSystemSettingsTestCase,
                        dbusmock.DBusTestCase):
    """ Base class for reset settings tests"""

    def mock_for_factory_reset(self):
        self.mock_server = self.spawn_server(SYSTEM_IFACE, SYSTEM_SERVICE_OBJ,
                                             SYSTEM_IFACE, system_bus=True,
                                             stdout=subprocess.PIPE)
        # spawn_server does not wait properly
        # Reported as bug here: http://pad.lv/1350833
        sleep(2)
        self.sys_mock = dbus.Interface(self.dbus_con.get_object(
            SYSTEM_IFACE, SYSTEM_SERVICE_OBJ), dbusmock.MOCK_IFACE)

        self.sys_mock.AddMethod(SYSTEM_IFACE, 'FactoryReset', '', '', '')

    @classmethod
    def setUpClass(klass):
        klass.start_system_bus()
        klass.dbus_con = klass.get_dbus(True)

    def setUp(self):
        self.mock_for_factory_reset()
        super(ResetBaseTestCase, self).setUp()
        self.reset_page = self.main_view.go_to_reset_phone()

    def tearDown(self):
        self.mock_server.terminate()
        self.mock_server.wait()
        super(ResetBaseTestCase, self).tearDown()


class SecurityBaseTestCase(LomiriSystemSettingsOfonoTestCase):
    """ Base class for security and privacy settings tests"""

    def setUp(self):
        super(SecurityBaseTestCase, self).setUp()
        self.security_page = self.main_view.go_to_security_page()

    def tearDown(self):
        super(SecurityBaseTestCase, self).tearDown()


class LanguageBaseTestCase(LomiriSystemSettingsTestCase,
                           dbusmock.DBusTestCase):
    """ Base class for language settings tests"""

    def mock_loginmanager(self):
        self.mock_server = self.spawn_server(LM_SERVICE, LM_PATH,
                                             LM_IFACE, system_bus=True,
                                             stdout=subprocess.PIPE)
        # spawn_server does not wait properly
        # Reported as bug here: http://pad.lv/1350833
        sleep(2)
        self.session_mock = dbus.Interface(self.dbus_con.get_object(
            LM_SERVICE, LM_PATH), dbusmock.MOCK_IFACE)

        self.session_mock.AddMethod(LM_IFACE, 'Reboot', 'b', '', '')

    @classmethod
    def setUpClass(klass):
        klass.start_system_bus()
        klass.dbus_con = klass.get_dbus(True)

    def setUp(self):
        self.mock_loginmanager()

        super(LanguageBaseTestCase, self).setUp()
        self.language_page = self.main_view.go_to_language_page()

    def tearDown(self):
        self.mock_server.terminate()
        self.mock_server.wait()
        super(LanguageBaseTestCase, self).tearDown()


class LomiriSystemSettingsVpnTestCase(LomiriSystemSettingsTestCase,
                                      dbusmock.DBusTestCase):
    """Base class for tests that tests the vpn functionality."""
    connectivity_parameters = {}
    indicatornetwork_parameters = {}

    @classmethod
    def setUpClass(cls):
        cls.session_con = cls.get_dbus(False)

        cls.start_system_bus()

        super(LomiriSystemSettingsVpnTestCase, cls).setUpClass()

    def setUp(self):
        if is_process_running(INDICATOR_NETWORK):
            _stop_process(INDICATOR_NETWORK)
            self.addCleanup(_start_process, INDICATOR_NETWORK)

        ctv_tmpl = os.path.join(os.path.dirname(__file__), 'connectivity.py')
        (self.ctv_mock, self.obj_ctv) = self.spawn_server_template(
            ctv_tmpl, parameters=self.connectivity_parameters,
            stdout=subprocess.PIPE)

        self.ctv_private = dbus.Interface(
            self.session_con.get_object(CTV_IFACE, CTV_PRIV_OBJ),
            'org.freedesktop.DBus.Properties')

        super(LomiriSystemSettingsVpnTestCase, self).setUp()

    def tearDown(self):
        self.ctv_mock.terminate()
        self.ctv_mock.wait()
        super(LomiriSystemSettingsVpnTestCase, self).tearDown()

    @classmethod
    def tearDownClass(cls):
        if dbusmock.DBusTestCase.system_bus_pid is not None:
            cls.stop_dbus(dbusmock.DBusTestCase.system_bus_pid)
            del os.environ['DBUS_SYSTEM_BUS_ADDRESS']
            dbusmock.DBusTestCase.system_bus_pid = None
        if dbusmock.DBusTestCase.session_bus_pid is not None:
            cls.stop_dbus(dbusmock.DBusTestCase.session_bus_pid)
            del os.environ['DBUS_SESSION_BUS_ADDRESS']
            dbusmock.DBusTestCase.session_bus_pid = None
        super(LomiriSystemSettingsVpnTestCase, cls).tearDownClass()


class VpnBaseTestCase(LomiriSystemSettingsVpnTestCase):

    def setUp(self):
        super(VpnBaseTestCase, self).setUp()
        # Required since this test needs to dismiss the OSK.
        self.useFixture(EnvironmentVariable("UITK_USE_MALIIT", "1"))
        self.vpn_page = self.main_view.go_to_vpn_page()

    def get_vpn_connection_object(self, path):
        return dbus.Interface(
            self.session_con.get_object(CTV_IFACE, path),
            'org.freedesktop.DBus.Properties'
        )


class WifiBaseTestCase(LomiriSystemSettingsTestCase,
                       dbusmock.DBusTestCase):
    """ Base class for wifi settings tests"""

    indicatornetwork_parameters = {}

    @classmethod
    def setUpClass(cls):
        cls.start_system_bus()
        cls.dbus_con = cls.get_dbus(True)
        cls.session_con = cls.get_dbus(False)

        template = os.path.join(os.path.dirname(__file__), 'networkmanager.py')
        (cls.p_mock, cls.obj_nm) = cls.spawn_server_template(
            template, stdout=subprocess.PIPE)
        super(WifiBaseTestCase, cls).setUpClass()

    def setUp(self, panel=None):
        if is_process_running(INDICATOR_NETWORK):
            _stop_process(INDICATOR_NETWORK)
            self.addCleanup(_start_process, INDICATOR_NETWORK)

        inetwork = os.path.join(
            os.path.dirname(__file__), 'indicatornetwork.py'
        )
        (self.inetwork_mock, self.obj_inetwork) = self.spawn_server_template(
            inetwork, parameters=self.indicatornetwork_parameters,
            stdout=subprocess.PIPE)

        self.obj_nm.Reset()

        # Add a mock NetworkManager environment so we get consistent results
        self.device_path = self.obj_nm.AddWiFiDevice('test0', 'Barbaz', 1)
        self.device_mock = dbus.Interface(self.dbus_con.get_object(
            NM_SERVICE, self.device_path),
            dbusmock.MOCK_IFACE)

        self.ap_mock = self.create_access_point(
            'test_ap', 'test_ap',
            security=NM80211ApSecurityFlags.NM_802_11_AP_SEC_KEY_MGMT_PSK
        )

        # Required since this test needs to dismiss the OSK.
        self.useFixture(EnvironmentVariable("UITK_USE_MALIIT", "1"))

        super(WifiBaseTestCase, self).setUp(panel)
        if panel:
            self.wifi_page = self.main_view.wait_select_single(
                objectName='wifiPage'
            )
        else:
            self.wifi_page = self.main_view.go_to_wifi_page()

        self.wifi_page._scroll_to_and_click = \
            self.main_view.scroll_to_and_click

    def tearDown(self):
        self.inetwork_mock.terminate()
        self.inetwork_mock.wait()
        super(WifiBaseTestCase, self).tearDown()

    def create_access_point(self, name, ssid, security=None):
        """Creates access point.

        :param name: Name of access point
        :param ssid: SSID of access point
        :param security: Either None, or a NM80211ApSecurityFlags

        :returns: Access point

        """
        if security is None:
            security = NM80211ApSecurityFlags.NM_802_11_AP_SEC_NONE

        return self.obj_nm.AddAccessPoint(
            self.device_path, name, ssid, self.random_mac_address(),
            InfrastructureMode.NM_802_11_MODE_INFRA, 2425, 5400, 82, security)

    def random_mac_address(self):
        """Returns a random Mac Address"""
        mac = [0x00, 0x16, 0x3e, random.randint(0x00, 0x7f),
               random.randint(0x00, 0xff), random.randint(0x00, 0xff)]
        return ':'.join(map(lambda x: "%02x" % x, mac))


class WifiWithSSIDBaseTestCase(WifiBaseTestCase):
    """ Class for Wi-Fi settings tests launches with an SSID."""

    ssid = None

    def setUp(self, panel=None):
        super(WifiWithSSIDBaseTestCase, self).setUp(
            panel='settings:///wifi/?ssid=%s' % self.ssid
        )



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

import dateutil.parser
import os
import subprocess
import unittest

from gi.repository import GLib

from autopilot.matchers import Eventually
from autopilot.platform import model
from testtools import skipIf
from testtools.matchers import Equals, NotEquals, Contains

from lomiri_system_settings.tests import (
    AboutBaseTestCase,
    AboutSystemImageBaseTestCase,
    AboutOfonoBaseTestCase,
    StorageBaseTestCase,
    LicenseBaseTestCase
)
from lomiri_system_settings.utils.i18n import ugettext as _

import dbus


class AboutTestCase(AboutBaseTestCase):

    """Tests for About this phone Page."""

    def _get_os_name(self):
        os_id = subprocess.check_output(
            ['lsb_release', '-is'], universal_newlines=True)
        os_release = subprocess.check_output(
            ['lsb_release', '-rs'], universal_newlines=True)

        return '{} {}'.format(os_id.strip(), os_release.strip())

    def _get_device_serial_number(self):
        try:
            return subprocess.check_output(
                ['getprop', 'ro.serialno'], universal_newlines=True).strip()
        except OSError:
            # getprop is only available on android hardware.
            return None

    def _get_device_manufacturer_and_model(self):
        if model() == 'Desktop':
            manufacturer = open(
                '/sys/devices/virtual/dmi/id/sys_vendor'
            ).read().strip()
            hw_model = open(
                '/sys/devices/virtual/dmi/id/product_name'
            ).read().strip()
        else:
            manufacturer = subprocess.check_output(
                ['getprop', 'ro.product.manufacturer'],
                universal_newlines=True
            ).strip()
            hw_model = subprocess.check_output(
                ['getprop', 'ro.product.model'],
                universal_newlines=True
            ).strip()

        return '{} {}'.format(manufacturer, hw_model)

    def test_device_with_serial_number_must_display_it(self):
        """Checks whether the UI is showing the correct serial number."""
        device_serial = self._get_device_serial_number()
        if not device_serial:
            self.skipTest('The device has no serial number.')
        else:
            self.assertTrue(self.about_page.is_serial_visible())
            displayed_serial = self.about_page.get_serial()
            self.assertThat(
                displayed_serial, Equals(device_serial))

    def test_device_without_serial_must_not_display_it(self):
        device_serial = self._get_device_serial_number()
        if device_serial:
            self.skipTest('The device has serial number.')
        else:
            self.assertFalse(self.about_page.is_serial_visible())

    def test_settings_show_correct_version_of_the_os(self):
        """Ensure the UI is showing the correct version of the OS."""
        device_os_version = self._get_os_name()
        displayed_os_info = self.about_page.get_os_information()
        # TODO: find a way to check the image build number as well
        # -- om26er 10-03-2014
        self.assertTrue(device_os_version in displayed_os_info)

    @skipIf(subprocess.call(
        ['which', 'getprop'], stdout=subprocess.PIPE) != 0,
        'program "getprop" not found'
    )
    def test_hardware_name(self):
        """Ensure the UI is showing the correct device name."""
        displayed_device_name = self.about_page.get_device_name()
        device_name_from_getprop = self._get_device_manufacturer_and_model()

        self.assertEquals(displayed_device_name, device_name_from_getprop)


class AboutOfonoTestCase(AboutOfonoBaseTestCase):
    def _get_imei_from_dbus(self):
        bus = self.get_dbus(system_bus=True)
        try:
            manager = dbus.Interface(
                bus.get_object('org.ofono', '/'), 'org.ofono.Manager'
            )
        except dbus.exceptions.DBusException:
            # oFono interface not found, probably its a desktop.
            return None

        modems = manager.GetModems()
        for path, properties in modems:
            return properties['Serial'] if 'Serial' in properties else None

    def test_device_with_imei_must_display_it(self):
        """Checks whether the UI is exposing the right IMEI."""
        device_imei = self._get_imei_from_dbus()
        if not device_imei:
            self.skipTest('The device has no imei.')
        else:
            self.assertTrue(self.about_page.is_imei_visible())
            displayed_imei = self.about_page.get_imei()
            self.assertThat(displayed_imei, Equals(device_imei))


class AboutSystemImageTestCase(AboutSystemImageBaseTestCase):

    def _get_system_image_iface(self):
        bus = self.get_dbus(system_bus=True)
        service = bus.get_object('com.ubports.SystemImage', '/Service')
        iface = dbus.Interface(service, 'com.ubports.SystemImage')
        return iface.Info()

    def _get_last_updated_date(self):
        info = self._get_system_image_iface()[3]

        if info == 'Unknown':
            return _('Never')
        else:
            return dateutil.parser.parse(info.split()[0]).date()

    def test_last_updated(self):
        """Checks whether Last Updated info is correct."""
        last_updated_date_displayed = \
            dateutil.parser.parse(
                self.about_page.get_last_updated_date()).date()
        self.assertEquals(
            last_updated_date_displayed, self._get_last_updated_date())

    def test_non_ota_version(self):
        """Checks whether a non-ota release gets an rev number."""
        os_item = self.about_page.wait_select_single(objectName='osItem')
        self.assertThat(os_item.value, Contains(' (r42)'))

    def test_check_for_updates(self):
        """
        Checks whether clicking on Check for Updates brings us
        to the Updates page.
        """
        system_updates_page = self.about_page.go_to_check_for_updates()
        self.assertThat(
            system_updates_page.visible, Eventually(Equals(True)))


class AboutOtaTestCase(AboutSystemImageBaseTestCase):
    systemimage_parameters = {
        'version_detail': 'ubuntu=20150123.1,device=20153344.1,\
            custom=201594834.1,version=257,tag=OTA-100'
    }

    def test_ota_version(self):
        """Checks whether a stable release gets an OTA number."""
        os_item = self.about_page.wait_select_single(objectName='osItem')
        self.assertThat(os_item.value, Contains(' (OTA-100)'))


class StorageTestCase(StorageBaseTestCase):
    """ Tests for Storage """

    def _get_space_by_directory(self, dir_name):
        if dir_name == 'Music':
            location = GLib.get_user_special_dir(
                GLib.UserDirectory.DIRECTORY_MUSIC
            )
        elif dir_name == 'Videos':
            location = GLib.get_user_special_dir(
                GLib.UserDirectory.DIRECTORY_VIDEOS
            )
        elif dir_name == 'Pictures':
            location = GLib.get_user_special_dir(
                GLib.UserDirectory.DIRECTORY_PICTURES
            )
        else:
            raise ValueError(
                '{} directory not handled by this fuction, you need to enhance'
                ' this function to handle that directory.'.format(dir_name)
            )

        if not os.path.exists(location):
            self.skipTest('glib directory {} does not exist'.format(dir_name))

        output = subprocess.check_output(['du', '--block-size=1', location])
        disk_space = output.split()[len(output.split()) - 2]
        return disk_space

    def test_disk(self):
        """ Checks whether disk item is available """
        disk_item = self.storage_page.select_single(
            objectName='diskItem'
        )
        self.assertThat(disk_item.text, Equals('Total storage'))

    def test_space(self):
        """ Checks whether storage item is available """
        self.assert_space_item('storageItem', _('Free space'))

    def test_space_ubuntu(self):
        """ Checks storage item """
        self.assert_space_item('usedByUbuntuItem', _('Used by Ubuntu'))

    @unittest.skip(
        'Disk calculation can take a while depending on different factors '
        'we dont want to wait for it to calculate.'
    )
    def test_space_used_by_movies(self):
        """ Checks whether space shown to be used by movies is
        correct. """
        movie_space = self._get_space_by_directory('Videos')
        movie_space_in_ui = self.get_storage_space_used_by_category(
            'moviesItem'
        )

        self.assertThat(movie_space_in_ui, Eventually(Equals(movie_space)))

    @unittest.skip(
        'Disk calculation can take a while depending on different factors '
        'we dont want to wait for it to calculate.'
    )
    def test_space_used_by_music(self):
        """ Checks whether space shown to be used by music is
        correct. """
        music_space = self._get_space_by_directory('Music')
        music_space_in_ui = self.get_storage_space_used_by_category(
            'audioItem'
        )

        self.assertThat(music_space_in_ui, Eventually(Equals(music_space)))

    @unittest.skip(
        'Disk calculation can take a while depending on different factors '
        'we dont want to wait for it to calculate.'
    )
    def test_space_used_by_pictures(self):
        """ Checks whether space shown to be used by pictures is
        correct. """
        pictures_space = self._get_space_by_directory('Pictures')
        pictures_space_in_ui = self.get_storage_space_used_by_category(
            'picturesItem'
        )

        self.assertThat(
            pictures_space_in_ui, Eventually(Equals(pictures_space))
        )

    def test_space_other_files(self):
        """ Checks whether space item is available """
        self.assert_space_item('otherFilesItem', _('Other files'))

    def test_space_used_by_apps(self):
        """ Checks whether space item is available """
        self.assert_space_item('usedByAppsItem', _('Used by apps'))

    def test_installed_apps(self):
        """ Checks whether Installed Apps list is available """
        installed_apps_list_view = self.storage_page.select_single(
            objectName='installedAppsListView'
        )
        self.assertThat(installed_apps_list_view, NotEquals(None))


class LicenseTestCase(LicenseBaseTestCase):

    """Tests for Licenses."""

    def test_open_licenses_page(self):
        """Check whether Storage page is available."""
        # FIXME this is not a good Autopilot tests. It would be better if it
        # opens one of the licenses and checks that it is visible.
        # --elopio - 2014-07-02
        self.assertThat(self.licenses_page.active, Equals(True))
